#!/usr/bin/env python

# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2015 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See http://swift.org/LICENSE.txt for license information
# See http://swift.org/CONTRIBUTORS.txt for the list of Swift project authors

# This is the script that is in charge of building PlaygroundLogger on Linux

import sys
import os
import os.path
import subprocess
import argparse
import colorama
import errno
import glob
import shutil

PROJECT_NAME = 'PlaygroundLogger'
SOURCE_DIR = os.path.dirname(os.path.abspath(__file__))

def formatted_msg(color,msg):
    if color:
        print "%s-build: %s%s%s" % (PROJECT_NAME,color,msg,colorama.Fore.RESET)
    else:
        print "%s-build: %s" % (PROJECT_NAME,msg)

def warning(msg):
    formatted_msg(colorama.Fore.YELLOW,msg)

def error(msg):
    formatted_msg(colorama.Fore.RED,msg)

def note(msg):
    formatted_msg(None,msg)

class BuildError(BaseException):
    def __init__(self):
        pass
    
    def __nonzero__(self):
        return False
    
    def __str__(self):
        return 'unknown build error'

class UncaughtExceptionBuildError(BuildError):
    def __init__(self, err):
        self.err = err
    
    def __str__(self):
        return 'uncaught exception: %s' % (self.err)

class NonZeroExitBuildError(BuildError):
    def __init__(self, cmd, err):
        self.cmd = cmd
        self.err = err

    def __str__(self):
        return '"%s" did not exit cleanly\n  exit-code: %d\n  stdout:\n%s' % (self.cmd,self.err.returncode,self.err.output)

def shell_exec(command):
    formatted_msg(None, 'running %s' % command)
    try:
        output = subprocess.check_output(command, shell=True, stderr=subprocess.STDOUT)
        note(output)
    except subprocess.CalledProcessError as fail:
        raise NonZeroExitBuildError(command,fail)

class BuildStyle(object):
    Debug = 1
    Release = 2

    @classmethod
    def fromString(cls, txt):
        if txt == 'release': return BuildStyle.Release
        if txt != 'debug': warning('selected build style %s is not debug or release - defaulting to debug' % txt)
        return BuildStyle.Debug

    @classmethod
    def toCommandLineOption(cls,self):
        if self == BuildStyle.Debug: return '-g -Onone'
        if self == BuildStyle.Release: return '-O'
        return None

class Settings(object):
    def __init__(self):
        pass
    
    def parse(self, args):
        self.buildStyle = BuildStyle.fromString(args.build_style)
        self.swiftCompiler = os.path.abspath(args.swiftc)
        if args.foundation:
            self.foundation = os.path.abspath(args.foundation)
        else:
            X = os.path.split(os.path.abspath(os.path.join(os.path.dirname(self.swiftCompiler),"..")))
            self.foundation = os.path.join(X[0],X[1].replace('swift-','foundation-'),'Foundation')
            note('foundation defaulted to "%s" based on compiler path' % self.foundation)
        if args.build_dir:
            self.buildDir = os.path.abspath(args.build_dir)
        else:
            self.buildDir = "/tmp"
            note('build-dir defaulted to /tmp')
        if args.swift_build_dir:
            self.swiftBuildDir = os.path.abspath(args.swift_build_dir)
        else:
            self.swiftBuildDir = os.path.abspath(os.path.join(os.path.dirname(self.swiftCompiler),'..'))
            note('swift-build-dir defaulted to "%s" based on compiler path' % self.swiftBuildDir)
        self.swiftAutolinkExtractPath = os.path.abspath(os.path.join(self.swiftBuildDir,'bin','swift-autolink-extract'))
        if args.module_path:
            self.moduleInstallPath = os.path.abspath(args.module_path)
        else:
            self.moduleInstallPath = None
        if args.lib_path:
            self.libraryInstallPath = os.path.abspath(args.lib_path)
        else:
            self.libraryInstallPath = None
        if not os.path.exists(self.swiftCompiler):
            error('%s not a valid compiler path' % self.swiftCompiler)
            sys.exit(1)
        if not os.path.exists(self.foundation):
            error('%s not a valid Foundation path' % self.foundation)
            sys.exit(1)
        if not os.path.exists(self.swiftBuildDir):
            error('%s not a valid swift build path' % self.swiftBuildDir)
            sys.exit(1)
        if (self.moduleInstallPath and not self.libraryInstallPath) or (self.libraryInstallPath and not self.moduleInstallPath):
            warning('both module and lib path required for installation - not installing')

    def __str__(self):
        s = ""
        for x in self.__dict__:
            s += "%s --> %s\n" % (x,self.__dict__[x])
        return s

SETTINGS = Settings()

class Target(object):
    def __init__(self, settings):
        self.settings = settings
    
    def description(self):
        return 'You should never see this'
    
    def build(self):
        return True

class CreateBuildDirectory(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'create the destination path'
    
    @classmethod
    def mkdirp(cls,path):
        try:
            os.makedirs(path)
        except OSError as exc:
            if exc.errno == errno.EEXIST and os.path.isdir(path):
                pass
            else:
                raise
    
    def build(self):
        CreateBuildDirectory.mkdirp(self.settings.buildDir)
        return True

class GenerateBOM(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'ingest PlaygroundLogger.bom'
    
    @classmethod
    def platform(cls):
        if sys.platform.startswith('linux'): return "linux"
        return sys.platform
    
    @classmethod
    def processLine(cls, line):
        if line.startswith('#'): return ''
        return line.strip()
    
    def build(self):
        self.settings.BOMFilePath = os.path.join(SOURCE_DIR,PROJECT_NAME,"%s-%s.bom" % (PROJECT_NAME,GenerateBOM.platform()))
        if os.path.exists(self.settings.BOMFilePath):
            self.settings.bom = set([GenerateBOM.processLine(line) for line in open(self.settings.BOMFilePath)])
        else:
            self.settings.bom = set()

class BuildObjectFile(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'compile PlaygroundLogger.o'
    
    def build(self):
        self.settings.objectFilePath = "%s.o" % (os.path.join(self.settings.buildDir,PROJECT_NAME))
        self.settings.moduleFilePath = "%s.swiftmodule" % (os.path.join(self.settings.buildDir,PROJECT_NAME))
        sources = glob.glob(os.path.join(SOURCE_DIR,PROJECT_NAME,"*.swift"))
        command = '"%s" %s -I "%s" -I "%s" -L "%s" -Xlinker -rpath -Xlinker "%s" -parse-as-library -emit-object -module-name %s -o "%s" -force-single-frontend-invocation -module-link-name %s -emit-module-path "%s" ' % \
                                               (self.settings.swiftCompiler,
                                                BuildStyle.toCommandLineOption(self.settings.buildStyle),
                                                self.settings.foundation,
                                                os.path.join(self.settings.foundation,'usr','lib','swift'),
                                                self.settings.foundation,
                                                self.settings.foundation,
                                                PROJECT_NAME,
                                                self.settings.objectFilePath,
                                                PROJECT_NAME,
                                                self.settings.moduleFilePath)
        for source in sources:
            if os.path.basename(source) in self.settings.bom:
                warning('"%s" skipped due to being present in the BOM file' % source)
            else:
                command += "%s " % source
        
        shell_exec(command)

class CreateAutolinkExtractFile(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'compile PlaygroundLogger.autolink'

    def build(self):
        self.settings.autolinkFilePath = "%s.autolink" % (os.path.join(self.settings.buildDir,PROJECT_NAME))
        command = '"%s" "%s" -o "%s"' % (self.settings.swiftAutolinkExtractPath,
                                         self.settings.objectFilePath,
                                         self.settings.autolinkFilePath)
        shell_exec(command)

class CreateModulewrapObjectFile(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'modulewrap into PlaygroundLoggerMV.o'
        
    def build(self):
        self.settings.modulewrapFilePath = "%sMV.o" % (os.path.join(self.settings.buildDir,PROJECT_NAME))
        command = '"%s" -modulewrap "%s" -o "%s"' % (self.settings.swiftCompiler,
                                                     self.settings.moduleFilePath,
                                                     self.settings.modulewrapFilePath)
        shell_exec(command)

class LinkDynamicLibrary(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'link libPlaygroundLogger.so'
    
    def build(self):
        self.settings.dylibPath = os.path.join(self.settings.buildDir,"lib%s.so" % PROJECT_NAME)
        command = 'clang "%s" "%s" -shared @"%s" -o "%s" -L "%s" -Wl,--no-undefined -Wl-soname,lib%s.so -L "%s" -lswiftCore -lFoundation -lswiftStdlibUnittest -lswiftGlibc -Xlinker -T "%s"' % \
                            (self.settings.objectFilePath,
                             self.settings.modulewrapFilePath,
                             self.settings.autolinkFilePath,
                             self.settings.dylibPath,
                             self.settings.foundation,
                             PROJECT_NAME,
                             os.path.join(self.settings.swiftBuildDir,'lib','swift','linux'),
                             os.path.join(self.settings.swiftBuildDir,'lib','swift','linux','x86_64','swift.ld'))

        shell_exec(command)

class BuildTestDriver(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'build PlaygroundLogger_TestDriver'
    
    def build(self):
        self.settings.testDriverPath = os.path.join(self.settings.buildDir,"%s_TestDriver" % PROJECT_NAME)
        sources = glob.glob(os.path.join(SOURCE_DIR,"%s_TestDriver" % PROJECT_NAME,"*.swift"))
        command = '"%s" %s -o "%s" -I "%s" -L "%s" -I "%s" -L "%s" -I "%s" -lFoundation -lPlaygroundLogger -Xlinker -rpath -Xlinker "%s" -Xlinker -rpath -Xlinker "%s" ' % \
                    (self.settings.swiftCompiler,
                     BuildStyle.toCommandLineOption(self.settings.buildStyle),
                     self.settings.testDriverPath,
                     self.settings.buildDir,
                     self.settings.buildDir,
                     self.settings.foundation,
                     self.settings.foundation,
                     os.path.join(self.settings.foundation,'usr','lib','swift'),
                     self.settings.foundation,
                     self.settings.buildDir)

        for source in sources:
            command += "%s " % source

        shell_exec(command)

class RunTestDriver(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'run PlaygroundLogger_TestDriver'
    
    def build(self):
        shell_exec(self.settings.testDriverPath)

class Install(Target):
    def __init__(self, settings):
        Target.__init__(self, settings)
    
    def description(self):
        return 'install PlaygroundLogger'
    
    def build(self):
        if self.settings.moduleInstallPath and self.settings.libraryInstallPath:
            shutil.copy(self.settings.dylibPath,self.settings.libraryInstallPath)
            shutil.copy(self.settings.moduleFilePath,self.settings.moduleInstallPath)
        else:
            note('skipping installation')

TARGETS = []

def main():
    colorama.init()
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description="Build %s" % PROJECT_NAME)
    parser.add_argument("--swiftc",
                        help="path to the swift compiler",
                        metavar="PATH",
                        action="store",
                        dest="swiftc",
                        required=True,
                        default=None)
    parser.add_argument("--foundation",
                        help="path to the Foundation build",
                        metavar="PATH",
                        action="store",
                        dest="foundation",
                        required=False,
                        default=None)
    parser.add_argument("--build-dir",
                        help="path to the output build directory",
                        metavar="PATH",
                        action="store",
                        dest="build_dir",
                        required=False,
                        default=None)
    parser.add_argument("--swift-build-dir",
                        help="path to the swift build directory",
                        metavar="PATH",
                        action="store",
                        dest="swift_build_dir",
                        required=False,
                        default=None)
    parser.add_argument("--module-install-path",
                        help="location to install module files",
                        metavar="PATH",
                        action="store",
                        dest="module_path",
                        default=None)
    parser.add_argument("--library-install-path",
                        help="location to install shared library files",
                        metavar="PATH",
                        action="store",
                        dest="lib_path",
                        default=None)
    parser.add_argument("--debug",
                        help="builds for debug (the default)",
                        action="store_const",
                        dest="build_style",
                        const="debug",
                        default="debug")
    parser.add_argument("--release",
                        help="builds for release",
                        action="store_const",
                        dest="build_style",
                        const="release",
                        default="debug")
    args = parser.parse_args()

    SETTINGS.parse(args)
    print SETTINGS
    
    TARGETS.append(CreateBuildDirectory)
    TARGETS.append(GenerateBOM)
    TARGETS.append(BuildObjectFile)
    TARGETS.append(CreateAutolinkExtractFile)
    TARGETS.append(CreateModulewrapObjectFile)
    TARGETS.append(LinkDynamicLibrary)
    TARGETS.append(BuildTestDriver)
    TARGETS.append(RunTestDriver)
    TARGETS.append(Install)

    for ctarget in TARGETS:
        target = ctarget(SETTINGS)
        note("building '%s'" % target.description())
        try:
            status = target.build()
        except BuildError as b:
            status = b
        except Exception as e:
            status = UncaughtExceptionBuildError(e)
        if status or (status is None):
            note('built successfully')
        else:
            error('build error: %s' % status)
            sys.exit(1)

if __name__ == '__main__':
    main()



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # Member access
        self.expect_expr("C.Base1::m_base", result_type="int", result_value="11")
        self.expect_expr("C.Base2::m_base", result_type="int", result_value="12")
        self.expect_expr("C.m1", result_type="int", result_value="22")
        self.expect_expr("C.m2", result_type="int", result_value="33")
        self.expect_expr("C.m_final", result_type="int", result_value="44")

        # Virtual functions
        self.expect_expr("C.Base1::virt_base()", result_type="int", result_value="111")
        self.expect_expr("C.Base2::virt_base()", result_type="int", result_value="121")
        self.expect_expr("C.virt1()", result_type="int", result_value="3")
        self.expect_expr("C.virt2()", result_type="int", result_value="5")
        self.expect_expr("C.final_virt()", result_type="int", result_value="7")
        self.expect_expr("C.virt_common()", result_type="int", result_value="444")

        # Normal functions
        self.expect_expr("C.Base1::func_base()", result_type="int", result_value="112")
        self.expect_expr("C.Base2::func_base()", result_type="int", result_value="122")
        self.expect_expr("C.func1()", result_type="int", result_value="4")
        self.expect_expr("C.func2()", result_type="int", result_value="6")
        self.expect_expr("C.final_func()", result_type="int", result_value="8")
        self.expect_expr("C.func_common()", result_type="int", result_value="888")



"""
Test that the 'gui' displays the help window and basic UI.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class BasicGuiCommandTest(PExpectTest):
    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfCursesSupportMissing
    @skipIfLinux #rdar://problem/55757360
    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])
    def test_gui(self):
        self.build()

        self.launch(executable=self.getBuildArtifact("a.out"), dimensions=(100, 500))
        self.expect(
            'br set -f main.c -p "// Break here"', substrs=["Breakpoint 1", "address ="]
        )
        self.expect("run", substrs=["stop reason ="])

        escape_key = chr(27).encode()

        # Start the GUI.
        self.child.sendline("gui")

        # Check for GUI elements in the menu bar.
        self.child.expect_exact("Target")
        self.child.expect_exact("Process")
        self.child.expect_exact("Thread")
        self.child.expect_exact("View")
        self.child.expect_exact("Help")

        # Check the sources window.
        self.child.expect_exact("Sources")
        self.child.expect_exact("main")
        self.child.expect_exact("funky_var_name_that_should_be_rendered")

        # Check the variable window.
        self.child.expect_exact("Variables")
        self.child.expect_exact("(int) funky_var_name_that_should_be_rendered = 22")

        # Check the bar at the bottom.
        self.child.expect_exact("Frame:")

        # Press escape to quit the gui
        self.child.send(escape_key)

        self.expect_prompt()
        self.quit()



"""
Test setting a breakpoint by line and column.
"""

import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class BreakpointByLineAndColumnTestCase(TestBase):
    ## Skip gcc version less 7.1 since it doesn't support -gcolumn-info
    @skipIf(compiler="gcc", compiler_version=["<", "7.1"])
    def testBreakpointByLineAndColumn(self):
        self.build()
        src_file = lldb.SBFileSpec("main.cpp")
        line = (
            line_number("main.cpp", "At the beginning of a function name (col:50)") + 1
        )  # Next line after comment
        _, _, _, breakpoint = lldbutil.run_to_line_breakpoint(self, src_file, line, 50)
        self.expect("fr v did_call", substrs=["1"])
        in_then = False
        for i in range(breakpoint.GetNumLocations()):
            b_loc = breakpoint.GetLocationAtIndex(i).GetAddress().GetLineEntry()
            self.assertEqual(b_loc.GetLine(), line)
            in_then |= b_loc.GetColumn() == 50
        self.assertTrue(in_then)

    ## Skip gcc version less 7.1 since it doesn't support -gcolumn-info
    @skipIf(compiler="gcc", compiler_version=["<", "7.1"])
    def testBreakpointByLine(self):
        self.build()
        src_file = lldb.SBFileSpec("main.cpp")
        line = (
            line_number("main.cpp", "At the beginning of a function name (col:50)") + 1
        )  # Next line after comment
        _, _, _, breakpoint = lldbutil.run_to_line_breakpoint(self, src_file, line)
        self.expect("fr v did_call", substrs=["0"])
        in_condition = False
        for i in range(breakpoint.GetNumLocations()):
            b_loc = breakpoint.GetLocationAtIndex(i).GetAddress().GetLineEntry()
            self.assertEqual(b_loc.GetLine(), line)
            in_condition |= b_loc.GetColumn() < 30
        self.assertTrue(in_condition)

    @skipIfWindows
    ## Skip gcc version less 7.1 since it doesn't support -gcolumn-info
    @skipIf(compiler="gcc", compiler_version=["<", "7.1"])
    def testBreakpointByLineAndColumnNearestCode(self):
        self.build()

        patterns = [
            "In the middle of a function name (col:42)",
            "In the middle of the lambda declaration argument (col:23)",
            "Inside the lambda (col:26)",
        ]

        source_loc = []

        for pattern in patterns:
            line = line_number("main.cpp", pattern) + 1
            column = int(re.search("\(col:([0-9]+)\)", pattern).group(1))
            source_loc.append({"line": line, "column": column})

        target = self.createTestTarget()

        for loc in source_loc:
            src_file = lldb.SBFileSpec("main.cpp")
            line = loc["line"]
            column = loc["column"]
            indent = 0
            module_list = lldb.SBFileSpecList()

            valid_bpkt = target.BreakpointCreateByLocation(
                src_file, line, column, indent, module_list, True
            )
            self.assertTrue(valid_bpkt, VALID_BREAKPOINT)
            self.assertEqual(valid_bpkt.GetNumLocations(), 1)

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        nearest_column = [7, 17, 26]

        for idx, loc in enumerate(source_loc):
            bpkt = target.GetBreakpointAtIndex(idx)
            bpkt_loc = bpkt.GetLocationAtIndex(0)
            self.assertEqual(bpkt_loc.GetHitCount(), 1)
            self.assertSuccess(process.Continue())
            bpkt_loc_desc = lldb.SBStream()
            self.assertTrue(
                bpkt_loc.GetDescription(bpkt_loc_desc, lldb.eDescriptionLevelVerbose)
            )
            self.assertIn(
                "main.cpp:{}:{}".format(loc["line"], nearest_column[idx]),
                bpkt_loc_desc.GetData(),
            )
            bpkt_loc_addr = bpkt_loc.GetAddress()
            self.assertTrue(bpkt_loc_addr)

            list = target.FindCompileUnits(lldb.SBFileSpec("main.cpp", False))
            # Executable has been built just from one source file 'main.cpp',
            # so we may check only the first element of list.
            compile_unit = list[0].GetCompileUnit()

            found = False
            for line_entry in compile_unit:
                if line_entry.GetStartAddress() == bpkt_loc_addr:
                    self.assertEqual(line_entry.GetFileSpec().GetFilename(), "main.cpp")
                    self.assertEqual(line_entry.GetLine(), loc["line"])
                    self.assertEqual(line_entry.GetColumn(), nearest_column[idx])
                    found = True
                    break

            self.assertTrue(found)

        line = line_number("main.cpp", "// This is a random comment.")
        column = len("// This is a random comment.")
        indent = 2
        invalid_bpkt = target.BreakpointCreateByLocation(
            src_file, line, column, indent, module_list, False
        )
        self.assertTrue(invalid_bpkt, VALID_BREAKPOINT)
        self.assertEqual(invalid_bpkt.GetNumLocations(), 0)



"""
Test lldb Python commands.
"""


import lldb
from lldbsuite.test.lldbtest import *


class CommandScriptAliasTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_pycmd(self):
        self.runCmd("command script import tcsacmd.py")
        self.runCmd("command script add -f tcsacmd.some_command_here attach")

        # This is the function to remove the custom commands in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("command script delete attach", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # We don't want to display the stdout if not in TraceOn() mode.
        if not self.TraceOn():
            self.HideStdout()

        self.expect("attach a", substrs=["Victory is mine"])
        self.runCmd("command script delete attach")
        # this can't crash but we don't care whether the actual attach works
        self.runCmd("attach noprocessexistswiththisname", check=False)



"""Test the SBPlatform APIs."""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class SBPlatformAPICase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfRemote  # Remote environment not supported.
    def test_run(self):
        self.build()
        plat = lldb.SBPlatform.GetHostPlatform()

        os.environ["MY_TEST_ENV_VAR"] = "SBPlatformAPICase.test_run"

        def cleanup():
            del os.environ["MY_TEST_ENV_VAR"]

        self.addTearDownHook(cleanup)
        cmd = lldb.SBPlatformShellCommand(self.getBuildArtifact("a.out"))
        self.assertSuccess(plat.Run(cmd))
        self.assertIn("MY_TEST_ENV_VAR=SBPlatformAPICase.test_run", cmd.GetOutput())

    def test_SetSDKRoot(self):
        plat = lldb.SBPlatform("remote-linux")  # arbitrary choice
        self.assertTrue(plat)
        plat.SetSDKRoot(self.getBuildDir())
        self.dbg.SetSelectedPlatform(plat)
        self.expect("platform status", substrs=["Sysroot:", self.getBuildDir()])

    def test_SetCurrentPlatform_floating(self):
        # floating platforms cannot be referenced by name until they are
        # associated with a debugger
        floating_platform = lldb.SBPlatform("remote-netbsd")
        floating_platform.SetWorkingDirectory(self.getBuildDir())
        self.assertSuccess(self.dbg.SetCurrentPlatform("remote-netbsd"))
        dbg_platform = self.dbg.GetSelectedPlatform()
        self.assertEqual(dbg_platform.GetName(), "remote-netbsd")
        self.assertIsNone(dbg_platform.GetWorkingDirectory())

    def test_SetCurrentPlatform_associated(self):
        # associated platforms are found by name-based lookup
        floating_platform = lldb.SBPlatform("remote-netbsd")
        floating_platform.SetWorkingDirectory(self.getBuildDir())
        orig_platform = self.dbg.GetSelectedPlatform()

        self.dbg.SetSelectedPlatform(floating_platform)
        self.dbg.SetSelectedPlatform(orig_platform)
        self.assertSuccess(self.dbg.SetCurrentPlatform("remote-netbsd"))
        dbg_platform = self.dbg.GetSelectedPlatform()
        self.assertEqual(dbg_platform.GetName(), "remote-netbsd")
        self.assertEqual(dbg_platform.GetWorkingDirectory(), self.getBuildDir())



"""
Test that objective-c expression parser continues to work for optimized build.

Fixed a bug in the expression parser where the 'this'
or 'self' variable was not properly read if the compiler
optimized it into a register.
"""


import lldb
import re

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

# rdar://problem/9087739
# test failure: objc_optimized does not work for "-C clang -A i386"


class ObjcOptimizedTestCase(TestBase):
    myclass = "MyClass"
    mymethod = "description"
    method_spec = "-[%s %s]" % (myclass, mymethod)

    @expectedFailureAll(remote=True)
    def test_break(self):
        """Test 'expr member' continues to work for optimized build."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_symbol(
            self, self.method_spec, num_expected_locations=1, sym_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
            patterns=["frame.*0:.*%s %s" % (self.myclass, self.mymethod)],
        )

        self.expect("expression member", startstr="(int) $0 = 5")

        # <rdar://problem/12693963>
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()
        interp.HandleCommand("frame variable self", result)
        output = result.GetOutput()

        desired_pointer = "0x0"

        mo = re.search("0x[0-9a-f]+", output)

        if mo:
            desired_pointer = mo.group(0)

        self.expect(
            "expression (self)",
            substrs=[("(%s *) $1 = " % self.myclass), desired_pointer],
        )

        self.expect(
            "expression self->non_member",
            error=True,
            substrs=["does not have a member named 'non_member'"],
        )



# TestSwiftDWARFImporterC.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2019 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftDWARFImporterC(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    def build(self):
        include = self.getBuildArtifact('include')
        inputs = self.getSourcePath('Inputs')
        lldbutil.mkdir_p(include)
        import shutil
        for f in ['module.modulemap', 'c-header.h', 'submodule.h']:
            shutil.copyfile(os.path.join(inputs, f), os.path.join(include, f))

        super(TestSwiftDWARFImporterC, self).build()

        # Remove the header files to thwart ClangImporter.
        self.assertTrue(os.path.isdir(include))
        shutil.rmtree(include)

    @skipIf(archs=['ppc64le'], bugnumber='SR-10214')
    @swiftTest
    # This test needs a working Remote Mirrors implementation.
    @skipIf(oslist=['windows'])
    def test_dwarf_importer(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))
        lldbutil.check_variable(self,
                                target.FindFirstGlobalVariable("pureSwift"),
                                value="42")
        lldbutil.check_variable(self,
                                target.FindFirstGlobalVariable("point"),
                                typename='CModule.Point', num_children=2)
        self.expect("target variable point", substrs=["x = 1", "y = 2"])
        self.expect("target variable enumerator", substrs=[".yellow"])
        self.expect("target variable pureSwiftStruct", substrs=["pure swift"])
        self.expect("target variable swiftStructCMember",
                    substrs=["point", "x = 3", "y = 4",
                             "sub", "x = 1", "y = 2", "z = 3",
                             "swift struct c member"])
        self.expect("target variable typedef", substrs=["x = 5", "y = 6"])
        #self.expect("target variable union",
        #            substrs=["(DoubleLongUnion)", "long_val = 42"])
        self.expect("target variable fromSubmodule",
                    substrs=["(FromSubmodule)", "x = 1", "y = 2", "z = 3"])
        self.expect("target variable withPointer",
                    substrs=["(WithPointer)", "ptr = nil"])

    @skipIf(archs=['ppc64le'], bugnumber='SR-10214')
    @swiftTest
    # This test needs a working Remote Mirrors implementation.
    @skipIf(oslist=['windows'])
    def test_dwarf_importer_exprs(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))
        self.expect("expr union", substrs=["(DoubleLongUnion)", "long_val = 42"])
        lldbutil.check_variable(self,
                                target.FindFirstGlobalVariable("pureSwift"),
                                value="42")
        lldbutil.check_variable(self,
                                target.FindFirstGlobalVariable("point"),
                                typename='CModule.Point', num_children=2)
        self.expect("expr point", substrs=["x = 1", "y = 2"])
        self.expect("expr enumerator", substrs=[".yellow"])
        self.expect("expr pureSwiftStruct", substrs=["pure swift"])
        self.expect("expr swiftStructCMember",
                    substrs=["point", "x = 3", "y = 4",
                             "sub", "x = 1", "y = 2", "z = 3",
                             "swift struct c member"])
        self.expect("expr typedef", substrs=["x = 5", "y = 6"])
        self.expect("expr union", substrs=["(DoubleLongUnion)", "long_val = 42"])
        self.expect("expr fromSubmodule",
                    substrs=["(FromSubmodule)", "x = 1", "y = 2", "z = 3"])
        
    @skipIf(archs=['ppc64le'], bugnumber='SR-10214')
    @swiftTest
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    def test_negative(self):
        lldb.SBDebugger.MemoryPressureDetected()
        self.runCmd("settings set symbols.use-swift-dwarfimporter false")
        self.build()
        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))
        lldbutil.check_variable(self,
                                target.FindFirstGlobalVariable("point"),
                                typename="CModule.Point", num_children=2)
        # This can't be resolved.
        self.expect("expr swiftStructCMember", error=True)

        found = False
        import io
        logfile = io.open(log, "r", encoding='utf-8')
        for line in logfile:
            if "missing required module" in line:
                found = True
        self.assertTrue(found)

        process.Clear()
        target.Clear()
        lldb.SBDebugger.MemoryPressureDetected()



"""
Test things related to the fission debug information style where the main object
file contains a skeleton compile unit and the main debug info is in .dwo files.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os


class ExecTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_zero_dwo_id(self):
        """
        Test that we can load a .o file that has a skeleton compile unit
        with a DWO ID of zero. We do this by hacking up the yaml to emit
        zero as a DWO ID is both the .o file and .dwo file. Then we make
        sure we can resolve something in the debug information to verify
        that we were able to load the .dwo file corrrectly since that is
        the only place that has this information.
        """
        src_dir = self.getSourceDir()
        dwo_yaml_path = os.path.join(src_dir, "main.dwo.yaml")
        obj_yaml_path = os.path.join(src_dir, "main.o.yaml")
        dwo_path = self.getBuildArtifact("main.dwo")
        obj_path = self.getBuildArtifact("main.o")
        self.yaml2obj(dwo_yaml_path, dwo_path)
        self.yaml2obj(obj_yaml_path, obj_path)

        # We need the current working directory to be set to the build directory
        os.chdir(self.getBuildDir())
        # Create a target with the object file we just created from YAML
        target = self.dbg.CreateTarget(obj_path)
        self.assertTrue(target, VALID_TARGET)

        # Set a breakpoint by file and line, this doesn't require anything from
        # the .dwo file.
        bp = target.BreakpointCreateByLocation("main.cpp", 6)
        self.assertEqual(bp.GetNumLocations(), 1)
        bp_loc = bp.GetLocationAtIndex(0)
        self.assertTrue(bp_loc.IsValid())

        # We will use the address of the location to resolve the function "main"
        # to make sure we were able to open the .dwo file since this is the only
        # place that contains debug info for the function.
        self.assertTrue(bp_loc.GetAddress().GetFunction().IsValid())



import lldb


def stop_if_called_from_a(frame, bp_loc, dict):
    thread = frame.GetThread()
    process = thread.GetProcess()
    target = process.GetTarget()
    dbg = target.GetDebugger()

    # Perform synchronous interaction with the debugger.
    old_async = dbg.GetAsync()
    dbg.SetAsync(True)

    # We check the call frames in order to stop only when the immediate caller
    # of the leaf function c() is a().  If it's not the right caller, we ask the
    # command interpreter to continue execution.

    should_stop = True
    if thread.GetNumFrames() >= 2:
        if (
            thread.frames[0].function.name == "c"
            and thread.frames[1].function.name == "a"
        ):
            should_stop = True
        else:
            should_stop = False

    dbg.SetAsync(old_async)
    return should_stop




"""
Test that printing Swift generic types with C++ types parameters works
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftForwardInteropGenericWithCxxType(TestBase):

    @swiftTest
    def test(self):
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        self.expect('v classWrapper', substrs=['Wrapper<CxxClass>', 't', 'a1 = 10',
            'a2 = 20', 'a3 = 30'])
        self.expect('expr classWrapper', substrs=['Wrapper<CxxClass>', 't', 'a1 = 10',
            'a2 = 20', 'a3 = 30'])

        self.expect('v subclassWrapper', substrs=['Wrapper<CxxSubclass>', 't', 
            'CxxClass = (a1 = 10, a2 = 20, a3 = 30)', 'a4 = 40'])
        self.expect('expr subclassWrapper', substrs=['Wrapper<CxxSubclass>', 't', 
            'CxxClass = (a1 = 10, a2 = 20, a3 = 30)', 'a4 = 40'])



#!/usr/bin/env python
##===-- lui.py -----------------------------------------------*- Python -*-===##
##
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
##
##===----------------------------------------------------------------------===##


import curses

import lldb
import lldbutil

from optparse import OptionParser
import os
import signal
import sys

try:
    import queue
except ImportError:
    import Queue as queue

import debuggerdriver
import cui

import breakwin
import commandwin
import eventwin
import sourcewin
import statuswin

event_queue = None


def handle_args(driver, argv):
    parser = OptionParser()
    parser.add_option(
        "-p",
        "--attach",
        dest="pid",
        help="Attach to specified Process ID",
        type="int")
    parser.add_option(
        "-c",
        "--core",
        dest="core",
        help="Load specified core file",
        type="string")

    (options, args) = parser.parse_args(argv)

    if options.pid is not None:
        try:
            pid = int(options.pid)
            driver.attachProcess(ui, pid)
        except ValueError:
            print("Error: expecting integer PID, got '%s'" % options.pid)
    elif options.core is not None:
        if not os.path.exists(options.core):
            raise Exception(
                "Specified core file '%s' does not exist." %
                options.core)
        driver.loadCore(options.core)
    elif len(args) == 2:
        if not os.path.isfile(args[1]):
            raise Exception("Specified target '%s' does not exist" % args[1])
        driver.createTarget(args[1])
    elif len(args) > 2:
        if not os.path.isfile(args[1]):
            raise Exception("Specified target '%s' does not exist" % args[1])
        driver.createTarget(args[1], args[2:])


def sigint_handler(signal, frame):
    global debugger
    debugger.terminate()


class LLDBUI(cui.CursesUI):

    def __init__(self, screen, event_queue, driver):
        super(LLDBUI, self).__init__(screen, event_queue)

        self.driver = driver

        h, w = self.screen.getmaxyx()

        command_win_height = 20
        break_win_width = 60

        self.status_win = statuswin.StatusWin(0, h - 1, w, 1)
        h -= 1
        self.command_win = commandwin.CommandWin(
            driver, 0, h - command_win_height, w, command_win_height)
        h -= command_win_height
        self.source_win = sourcewin.SourceWin(driver, 0, 0,
                                              w - break_win_width - 1, h)
        self.break_win = breakwin.BreakWin(driver, w - break_win_width, 0,
                                           break_win_width, h)

        self.wins = [self.status_win,
                     # self.event_win,
                     self.source_win,
                     self.break_win,
                     self.command_win,
                     ]

        self.focus = len(self.wins) - 1  # index of command window;

    def handleEvent(self, event):
        # hack
        if isinstance(event, int):
            if event == curses.KEY_F10:
                self.driver.terminate()
            if event == 20:  # ctrl-T
                def foo(cmd):
                    ret = lldb.SBCommandReturnObject()
                    self.driver.getCommandInterpreter().HandleCommand(cmd, ret)
                foo('target create a.out')
                foo('b main')
                foo('run')
        super(LLDBUI, self).handleEvent(event)


def main(screen):
    signal.signal(signal.SIGINT, sigint_handler)

    global event_queue
    event_queue = queue.Queue()

    global debugger
    debugger = lldb.SBDebugger.Create()

    driver = debuggerdriver.createDriver(debugger, event_queue)
    view = LLDBUI(screen, event_queue, driver)

    driver.start()

    # hack to avoid hanging waiting for prompts!
    driver.handleCommand("settings set auto-confirm true")

    handle_args(driver, sys.argv)
    view.eventLoop()

if __name__ == "__main__":
    try:
        curses.wrapper(main)
    except KeyboardInterrupt:
        exit()




"""
Test that Swift types are displayed correctly in C++
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftBackwardInteropFormatActors(TestBase):

    @swiftTest
    def test_class(self):
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.cpp'))

        self.expect('v actor', substrs=['Actor', 'str = "Hello"'])



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftVariadicGenerics(TestBase):

    @skipUnlessDarwin
    @swiftTest
    def test(self):
        self.build()

        target,  process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec('a.swift'))

        # f1(args: a, b)
        self.expect("frame variable",
                    substrs=["Pack{(a.A, a.B)}", "args", "i = 23", "d = 2.71"])

        # Test that an expression can be set up in a variadic environment.
        self.expect("expr --bind-generic-types=true -- 0", substrs=["0"])
        self.expect("expr --bind-generic-types=false -- 0", substrs=["0"])
        # FIXME: crashes the compiler.
        #self.expect("expr --bind-generic-params=false -- (repeat each args)",
        #            substrs=["args"])

        # f2(us: a, vs: b)
        process.Continue()
        self.expect("frame variable",
                    substrs=["Pack{(a.A)}", "us", "i = 23",
                             "Pack{(a.B)}", "vs", "d = 2.71"])

        # f3(ts: a, b, more_ts: a, b)
        process.Continue()
        self.expect("frame variable",
                    substrs=["Pack{(a.A, a.B)}", "ts", "i = 23", "d = 2.71",
                             "Pack{(a.A, a.B)}", "more_ts", "i = 23", "d = 2.71"])

        # f4(uvs: (a, b), (a, b))
        process.Continue()
        self.expect("frame variable",
                    substrs=[""])

        # f5(ts: (a, b), (42, b))
        process.Continue()
        self.expect("frame variable",
                    substrs=[# FIXME: "Pack{(a.A, a.B), (Int, a,B)}",
                             "ts", "i = 23", "d = 2.71"
                             # FIXME: "42", "d = 2.71"
                    ])

        # f6(us: a, more_us: a, vs: b, b)
        process.Continue()
        self.expect("frame variable",
                    substrs=["Pack{(a.A)}", "us", "i = 23",
                             "Pack{(a.A)}", "more_us", "i = 23",
                             "Pack{(a.B, a.B)}", "vs", "d = 2.71", "d = 2.71"
                             ])

        # f7(us: a, vs: 1, b, more_us: a, more_vs: 2, b)
        process.Continue()
        self.expect("frame variable",
                    substrs=["Pack{(a.A)}", "us", "i = 23",
                             "Pack{(Int, a.B)}", "vs", "= 1", "d = 2.71",
                             "Pack{(a.A)}", "more_us", "i = 23",
                             "Pack{(Int, a.B)}", "more_vs", "= 2", "d = 2.71"
                    ])

        # f8(<specialized self>)
        process.Continue()
        # specialized global
        process.Continue()
        self.expect("target variable s",
                    substrs=["vals", "0 = 23", "1 = 2.71"])

        # f9(s: S<repeat each T>)
        process.Continue()
        self.expect("frame variable", substrs=["t", "0 = 23", "1 = 2.71"])

        # f10<each T>(args: repeat each T)
        process.Continue()
        self.expect(
            "frame variable", substrs=[
                "Pack{(a.A, a.B)}",
                "args",
                "i = 23",
                "d ="
                # FIXME: The wrong value for d is currently shown.
                #"d = 2.71"
            ]
        )



# TestSwiftTupleTypes.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test support for tuple types
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftTupleTypes(TestBase):
    @swiftTest
    def test_swift_tuple_types(self):
        """Test support for tuple types"""
        self.build()
        lldbutil.run_to_source_breakpoint(self, 'Set breakpoint here',
                                          lldb.SBFileSpec('main.swift'))

        self.expect("frame variable t --", substrs=[
            '(Int, Int, Int) t = ', '0 = 111', '1 = 222', '2 = 333'])
        self.runCmd("continue")

        self.expect("frame variable --raw-output --show-types tuple1",
                    substrs=['(length: Swift.Int, name: Swift.String) tuple1',
                             '(Swift.Int) length =',
                             'value = 123',
                             '(Swift.String) name ='])
        self.expect("frame variable --raw-output --show-types tuple2",
                    substrs=['(Swift.Int, Swift.String) tuple2',
                             '(Swift.Int) 0 =',
                             'value = 123',
                             '(Swift.String) 1 ='])
        self.expect("frame variable --raw-output --show-types tuple3",
                    substrs=['(Swift.Int, name: Swift.String) tuple3',
                             '(Swift.Int) 0 =',
                             'value = 123',
                             '(Swift.String) name ='])
        self.expect("frame variable --raw-output --show-types tuple4",
                    substrs=['(p1: ', 'Point, p2: ', '.Point) tuple4',
                             'Point) p1',
                             'FPIEEE32)', 'value = 1.25',
                             'FPIEEE32)', 'value = 2.125',
                             'Point) p2',
                             'FPIEEE32)', 'value = 4.5',
                             'FPIEEE32)', 'value = 8.75'])




import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os

class TestSwiftDWARFImporterFromDylib(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    # This test needs a working Remote Mirrors implementation.
    @skipIf(oslist=['linux', 'windows'])
    def test_dwarf_importer(self):
        self.build()
        #os.remove(self.getBuildArtifact('Foo.swiftmodule'))
        lldbutil.run_to_source_breakpoint(self, 'break here',
                                          lldb.SBFileSpec('main.swift'),
                                          extra_images=['Foo'])
        # This type can only be imported into Swift via DWARFImporter
        # and is not visible from the main module at all.
        self.expect("expr -- WrappingFromDylib()", substrs=['23'])



"""
Test the lldb disassemble command on each call frame when stopped on C's ctor.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class IterateFrameAndDisassembleTestCase(TestBase):
    def test_and_run_command(self):
        """Disassemble each call frame when stopped on C's constructor."""
        self.build()
        self.breakOnCtor()

        raw_output = self.res.GetOutput()
        frameRE = re.compile(
            r"""
                              ^\s\sframe        # heading for the frame info,
                              .*                # wildcard, and
                              0x[0-9a-f]{16}    # the frame pc, and
                              \sa.out`(.+)      # module`function, and
                              \s\+\s            # the rest ' + ....'
                              """,
            re.VERBOSE,
        )
        for line in raw_output.split(os.linesep):
            match = frameRE.search(line)
            if match:
                function = match.group(1)
                self.trace("line:", line)
                self.trace("function:", function)
                self.runCmd("disassemble -n '%s'" % function)

    @add_test_categories(["pyapi"])
    def test_and_python_api(self):
        """Disassemble each call frame when stopped on C's constructor."""
        self.build()
        self.breakOnCtor()

        # Now use the Python API to get at each function on the call stack and
        # disassemble it.
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)
        if self.TraceOn():
            for frame in thread.frames:
                function = frame.GetFunction()
                if function:
                    # Print the function header.
                    print()
                    print(function)
                    # Get all instructions for this function and print them out.
                    insts = function.GetInstructions(target)
                    for inst in insts:
                        print(inst)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.cpp.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def breakOnCtor(self):
        """Setup/run the program so it stops on C's constructor."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break on the ctor function of class C.
        bpno = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=-1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint %d." % (bpno)],
        )

        # This test was failing because we fail to put the C:: in front of constructore.
        # We should maybe make another testcase to cover that specifically, but we shouldn't
        # fail this whole testcase for an inessential issue.
        # We should be stopped on the ctor function of class C.
        # self.expect("thread backtrace", BACKTRACE_DISPLAYED_CORRECTLY,
        #  substrs = ['C::C'])



# TestSwiftInterfaceDSYM.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2019 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# -----------------------------------------------------------------------------

import lldb
import os
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftInterfaceDSYM(TestBase):
    @swiftTest
    @skipIf(archs=no_match("x86_64"))
    @skipIf(debug_info=no_match(["dsym"]))
    def test_dsym_swiftinterface(self):
        """Test that LLDB can import .swiftinterface files inside a .dSYM bundle."""
        self.build()
        self.assertFalse(os.path.isdir(self.getBuildArtifact("AA.dylib.dSYM")),
                        "dylib dsym exists")

        # The custom swift module cache location
        swift_mod_cache = self.getBuildArtifact("cache")
        self.assertTrue(os.path.isfile(self.getBuildArtifact(
            "a.out.dSYM/Contents/Resources/Swift/x86_64/AA.swiftinterface")),
            "dsymutil doesn't support Swift interfaces")
        # Delete the .swiftinterface form the build dir.
        os.remove(self.getBuildArtifact("AA.swiftinterface"))

        # Clear the swift module cache (populated by the Makefile build)
        shutil.rmtree(swift_mod_cache)
        import glob
        self.assertFalse(os.path.isdir(swift_mod_cache),
                         "module cache should not exist")

        # Update the settings to use the custom module cache location
        self.runCmd('settings set symbols.clang-modules-cache-path "%s"'
                    % swift_mod_cache)

        # Set a breakpoint in and launch the main executable
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift"))

        # Check we are able to access the public fields of variables whose
        # types are from the .swiftinterface-only dylibs
        var = self.frame().FindVariable("x")
        lldbutil.check_variable(self, var, False, typename="AA.MyPoint")

        child_y = var.GetChildMemberWithName("y") # MyPoint.y is public
        lldbutil.check_variable(self, child_y, False, value="0")

        # MyPoint.x isn't public, but LLDB can find it through type metadata.
        child_x = var.GetChildMemberWithName("x")
        self.assertTrue(child_x.IsValid())

        # Expression evaluation using types from the .swiftinterface only
        # dylibs should work too
        lldbutil.check_expression(
            self, self.frame(), "y.magnitudeSquared", "404", use_summary=False)
        lldbutil.check_expression(
            self, self.frame(), "MyPoint(x: 1, y: 2).magnitudeSquared", "5",
            use_summary=False)

        # Check the swift module cache was populated with the .swiftmodule
        # files of the loaded modules
        self.assertTrue(os.path.isdir(swift_mod_cache), "module cache exists")
        a_modules = glob.glob(os.path.join(swift_mod_cache, 'AA-*.swiftmodule'))
        self.assertEqual(len(a_modules), 1)

    @swiftTest
    @skipIf(archs=no_match("x86_64"))
    @skipIf(debug_info=no_match(["dsym"]))
    def test_sanity_negative(self):
        """without the swiftinterface the test should fail"""
        self.build()
        self.assertFalse(os.path.isdir(self.getBuildArtifact("AA.dylib.dSYM")),
                        "dylib dsym exists")

        swift_mod_cache = self.getBuildArtifact("cache")

        # Delete the .swiftinterfaces from the .dSYM bundle.
        shutil.rmtree(
            self.getBuildArtifact("a.out.dSYM/Contents/Resources/Swift"),
            ignore_errors=True)

        # Delete the .swiftinterface form the build dir.
        os.remove(self.getBuildArtifact("AA.swiftinterface"))
        shutil.rmtree(swift_mod_cache)
        self.runCmd('settings set symbols.clang-modules-cache-path "%s"'
                    % swift_mod_cache)

        # Set a breakpoint in and launch the main executable
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift"))

        # We should not find a type for x.
        var = self.frame().FindVariable("x")
        # This was true for SwiftASTContext, but
        # TypeSystemSwiftTyperef succeeds, because this test it only
        # prints the type *name*.
        self.assertEqual(var.GetTypeName(), "AA.MyPoint")
        # Evaluating an expression fails, though.
        self.expect("expression x", error=1)

    @swiftTest
    @skipIf(archs=no_match("x86_64"))
    @skipIf(debug_info=no_match(["dsym"]))
    def test_sanity_positive(self):
        """Test that the presence of a .swiftinterface doesn't shadow a
           .swiftmodule"""
        self.build(dictionary={'DYLIB_DSYM': 'YES'})
        self.assertTrue(os.path.isdir(self.getBuildArtifact("libAA.dylib.dSYM")),
                        "dylib dsym exists")

        swift_mod_cache = self.getBuildArtifact("cache")
        self.assertTrue(os.path.isdir("%s.dSYM/Contents/Resources/Swift"
                                      %self.getBuildArtifact()),
                        "dsymutil doesn't support Swift interfaces")

        shutil.rmtree(swift_mod_cache)
        self.runCmd('settings set symbols.clang-modules-cache-path "%s"'
                    % swift_mod_cache)

        # Set a breakpoint in and launch the main executable
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift"))
        self.expect('breakpoint set -f AA.swift -l 11')

        #
        # main.swift
        #

        # We *should* find a type for x.
        var = self.frame().FindVariable("x")
        lldbutil.check_variable(self, var, typename="AA.MyPoint")
        # MyPoint.x is private.
        child_x = var.GetChildMemberWithName("x")
        self.assertTrue(child_x.IsValid())

        # The expression evaluator sees the whole program, so it also
        # sees the private members.
        self.expect("expression x", substrs=["x = 10"])
        # FIXME: this doesn't work, the summary/value is null/null.
        #lldbutil.check_expression(
        #    self, frame, "x", "x = 10", use_summary=False)

        #
        # AA.swift
        #

        process.Continue()
        var = self.frame().FindVariable("self")
        lldbutil.check_variable(self, var, typename="AA.MyPoint")
        # MyPoint.x is private and we should still see it.
        child_x = var.GetChildMemberWithName("x")
        lldbutil.check_variable(self, child_x, value="10")
        self.expect("expression self", substrs=["x = 10"])




from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestTraceDumpInstructions(TraceIntelPTTestCaseBase):
    def testErrorMessages(self):
        # We first check the output when there are no targets
        self.expect(
            "thread trace dump instructions",
            substrs=[
                "error: invalid target, create a target using the 'target create' command"
            ],
            error=True,
        )

        # We now check the output when there's a non-running target
        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )

        self.expect(
            "thread trace dump instructions",
            substrs=["error: Command requires a current process."],
            error=True,
        )

        # Now we check the output when there's a running target without a trace
        self.expect("b main")
        self.expect("run")

        self.expect(
            "thread trace dump instructions",
            substrs=["error: Process is not being traced"],
            error=True,
        )

    def testRawDumpInstructionsInJSON(self):
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
            substrs=["intel-pt"],
        )

        self.expect(
            "thread trace dump instructions --raw --count 5 --forwards --json",
            substrs=[
                """[{"id":3,"loadAddress":"0x400511"}""",
                """{"id":7,"loadAddress":"0x400518"}""",
                """{"id":8,"loadAddress":"0x40051f"}""",
                """{"id":9,"loadAddress":"0x400529"}""",
                """{"id":10,"loadAddress":"0x40052d"}""",
            ],
        )

        self.expect(
            "thread trace dump instructions --raw --count 5 --forwards --pretty-json",
            substrs=[
                """[
  {
    "id": 3,
    "loadAddress": "0x400511"
  },
  {
    "id": 7,
    "loadAddress": "0x400518"
  },
  {
    "id": 8,
    "loadAddress": "0x40051f"
  },
  {
    "id": 9,
    "loadAddress": "0x400529"
  },
  {
    "id": 10,
    "loadAddress": "0x40052d"
  }
]"""
            ],
        )

    def testRawDumpInstructionsInJSONToFile(self):
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
            substrs=["intel-pt"],
        )

        outfile = os.path.join(self.getBuildDir(), "output.json")

        self.expect(
            "thread trace dump instructions --raw --count 5 --forwards --pretty-json --file "
            + outfile
        )

        with open(outfile, "r") as out:
            self.assertEqual(
                out.read(),
                """[
  {
    "id": 3,
    "loadAddress": "0x400511"
  },
  {
    "id": 7,
    "loadAddress": "0x400518"
  },
  {
    "id": 8,
    "loadAddress": "0x40051f"
  },
  {
    "id": 9,
    "loadAddress": "0x400529"
  },
  {
    "id": 10,
    "loadAddress": "0x40052d"
  }
]""",
            )

    def testRawDumpInstructions(self):
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
            substrs=["intel-pt"],
        )

        self.expect(
            "thread trace dump instructions --raw --count 21 --forwards",
            substrs=[
                """thread #1: tid = 3842849
    3: 0x0000000000400511
    7: 0x0000000000400518
    8: 0x000000000040051f
    9: 0x0000000000400529
    10: 0x000000000040052d
    11: 0x0000000000400521
    12: 0x0000000000400525
    13: 0x0000000000400529
    14: 0x000000000040052d
    15: 0x0000000000400521
    16: 0x0000000000400525
    17: 0x0000000000400529
    18: 0x000000000040052d
    19: 0x0000000000400521
    20: 0x0000000000400525
    21: 0x0000000000400529
    22: 0x000000000040052d
    23: 0x0000000000400521
    24: 0x0000000000400525
    25: 0x0000000000400529
    26: 0x000000000040052"""
            ],
        )

        # We check if we can pass count and skip
        self.expect(
            "thread trace dump instructions --count 5 --skip 6 --raw --forwards",
            substrs=[
                """thread #1: tid = 3842849
    7: 0x0000000000400518
    8: 0x000000000040051f
    9: 0x0000000000400529
    10: 0x000000000040052d
    11: 0x0000000000400521"""
            ],
        )

        self.expect(
            "thread trace dump instructions --count 5 --skip 6 --raw",
            substrs=[
                """thread #1: tid = 3842849
    21: 0x0000000000400529
    20: 0x0000000000400525
    19: 0x0000000000400521
    18: 0x000000000040052d
    17: 0x0000000000400529"""
            ],
        )

        # We check if we can pass count and skip and instruction id in hex
        self.expect(
            "thread trace dump instructions --count 5 --skip 6 --raw --id 0xE",
            substrs=[
                """thread #1: tid = 3842849
    8: 0x000000000040051f
    7: 0x0000000000400518
    3: 0x0000000000400511
    no more data"""
            ],
        )

        # We check if we can pass count and skip and instruction id in decimal
        self.expect(
            "thread trace dump instructions --count 5 --skip 6 --raw --id 14",
            substrs=[
                """thread #1: tid = 3842849
    8: 0x000000000040051f
    7: 0x0000000000400518
    3: 0x0000000000400511
    no more data"""
            ],
        )

        # We check if we can access the thread by index id
        self.expect(
            "thread trace dump instructions 1 --raw",
            substrs=[
                """thread #1: tid = 3842849
    26: 0x000000000040052d"""
            ],
        )

        # We check that we get an error when using an invalid thread index id
        self.expect(
            "thread trace dump instructions 10",
            error=True,
            substrs=['error: no thread with index: "10"'],
        )

    def testDumpFullInstructionsWithMultipleThreads(self):
        # We load a trace with two threads
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace_2threads.json")
        )

        # We print the instructions of a specific thread
        self.expect(
            "thread trace dump instructions 2 --count 2",
            substrs=[
                """thread #2: tid = 3842850
  a.out`main + 32 at main.cpp:4
    26: 0x000000000040052d    jle    0x400521                  ; <+20> at main.cpp:5
    25: 0x0000000000400529    cmpl   $0x3, -0x8(%rbp)"""
            ],
        )

        # We use custom --count and --skip, saving the command to history for later
        self.expect(
            "thread trace dump instructions 2 --count 2 --skip 2",
            inHistory=True,
            substrs=[
                """thread #2: tid = 3842850
  a.out`main + 28 at main.cpp:4
    25: 0x0000000000400529    cmpl   $0x3, -0x8(%rbp)
    24: 0x0000000000400525    addl   $0x1, -0x8(%rbp)"""
            ],
        )

        # We use a repeat command twice and ensure the previous count is used and the
        # start position moves with each command.
        self.expect(
            "",
            inHistory=True,
            substrs=[
                """thread #2: tid = 3842850
  a.out`main + 20 at main.cpp:5
    23: 0x0000000000400521    xorl   $0x1, -0x4(%rbp)
  a.out`main + 32 at main.cpp:4
    22: 0x000000000040052d    jle    0x400521                  ; <+20> at main.cpp:5"""
            ],
        )

        self.expect(
            "",
            inHistory=True,
            substrs=[
                """thread #2: tid = 3842850
  a.out`main + 28 at main.cpp:4
    21: 0x0000000000400529    cmpl   $0x3, -0x8(%rbp)
    20: 0x0000000000400525    addl   $0x1, -0x8(%rbp"""
            ],
        )

    def testInvalidBounds(self):
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json")
        )

        # The output should be work when too many instructions are asked
        self.expect(
            "thread trace dump instructions --count 20 --forwards",
            substrs=[
                """thread #1: tid = 3842849
  a.out`main + 4 at main.cpp:2
    3: 0x0000000000400511    movl   $0x0, -0x4(%rbp)
  a.out`main + 11 at main.cpp:4
    7: 0x0000000000400518    movl   $0x0, -0x8(%rbp)
    8: 0x000000000040051f    jmp    0x400529                  ; <+28> at main.cpp:4"""
            ],
        )

        # Should print no instructions if the position is out of bounds
        self.expect("thread trace dump instructions --skip 23", endstr="no more data\n")

        # Should fail with negative bounds
        self.expect("thread trace dump instructions --skip -1", error=True)
        self.expect("thread trace dump instructions --count -1", error=True)

    def testWrongImage(self):
        self.expect(
            "trace load "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace_bad_image.json")
        )
        self.expect(
            "thread trace dump instructions --forwards",
            substrs=[
                """thread #1: tid = 3842849
    ...missing instructions
    3: (error) no memory mapped at this address: 0x0000000000400511"""
            ],
        )

    def testWrongCPU(self):
        self.expect(
            "trace load "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace_wrong_cpu.json")
        )
        self.expect(
            "thread trace dump instructions --forwards",
            substrs=["error: unknown cpu"],
            error=True,
        )

    def testMultiFileTraceWithMissingModuleInJSON(self):
        self.expect(
            "trace load "
            + os.path.join(
                self.getSourceDir(), "intelpt-trace-multi-file", "multi-file-no-ld.json"
            )
        )

        self.expect(
            "thread trace dump instructions --count 4 --id 9 --forwards --pretty-json",
            substrs=[
                """[
  {
    "id": 9,
    "loadAddress": "0x40054b",
    "module": "a.out",
    "symbol": "foo()",
    "mnemonic": "jmp"
  },
  {
    "id": 10,
    "loadAddress": "0x400510",
    "module": "a.out",
    "symbol": null,
    "mnemonic": "pushq"
  },
  {
    "id": 11,
    "loadAddress": "0x400516",
    "module": "a.out",
    "symbol": null,
    "mnemonic": "jmpq"
  },
  {
    "id": 12,
    "error": "no memory mapped at this address: 0x00007ffff7df1950"
  },
  {
    "id": 16,
    "loadAddress": "0x400674",
    "module": "a.out",
    "symbol": "main",
    "mnemonic": "movl",
    "source": "/home/wallace/llvm-sand/external/llvm-project/lldb/test/API/commands/trace/intelpt-trace-multi-file/main.cpp",
    "line": 10,
    "column": 0
  }
]"""
            ],
        )

        self.expect(
            "thread trace dump instructions --count 4 --id 20 --forwards --pretty-json",
            substrs=[
                """[
  {
    "id": 20,
    "loadAddress": "0x400677",
    "module": "a.out",
    "symbol": "main",
    "mnemonic": "movl",
    "source": "/home/wallace/llvm-sand/external/llvm-project/lldb/test/API/commands/trace/intelpt-trace-multi-file/main.cpp",
    "line": 12,
    "column": 0
  },
  {
    "id": 21,
    "loadAddress": "0x40067a",
    "module": "a.out",
    "symbol": "main",
    "mnemonic": "addl",
    "source": "/home/wallace/llvm-sand/external/llvm-project/lldb/test/API/commands/trace/intelpt-trace-multi-file/main.cpp",
    "line": 12,
    "column": 0
  },
  {
    "id": 22,
    "loadAddress": "0x40067f",
    "module": "a.out",
    "symbol": "main",
    "mnemonic": "movl",
    "source": "/home/wallace/llvm-sand/external/llvm-project/lldb/test/API/commands/trace/intelpt-trace-multi-file/main.cpp",
    "line": 12,
    "column": 0
  },
  {
    "id": 26,
    "loadAddress": "0x400682",
    "module": "a.out",
    "symbol": "inline_function()",
    "mnemonic": "movl",
    "source": "/home/wallace/llvm-sand/external/llvm-project/lldb/test/API/commands/trace/intelpt-trace-multi-file/main.cpp",
    "line": 4,
    "column": 0
  }
]"""
            ],
        )

    def testMultiFileTraceWithMissingModule(self):
        self.expect(
            "trace load "
            + os.path.join(
                self.getSourceDir(), "intelpt-trace-multi-file", "multi-file-no-ld.json"
            )
        )

        # This instructions in this test covers the following flow:
        #
        # - The trace starts with a call to libfoo, which triggers the dynamic
        #   linker, but the dynamic linker is not included in the JSON file,
        #   thus the trace reports a set of missing instructions after
        #   instruction [6].
        # - Then, the dump continues in the next synchronization point showing
        #   a call to an inlined function, which is displayed as [inlined].
        # - Finally, a call to libfoo is performed, which invokes libbar inside.
        #
        # Whenever there's a line or symbol change, including the inline case, a
        # line is printed showing the symbol context change.
        #
        # Finally, the instruction disassembly is included in the dump.
        self.expect(
            "thread trace dump instructions --count 50 --forwards",
            substrs=[
                """thread #1: tid = 815455
  a.out`main + 15 at main.cpp:10
    3: 0x000000000040066f    callq  0x400540                  ; symbol stub for: foo()
  a.out`symbol stub for: foo()
    7: 0x0000000000400540    jmpq   *0x200ae2(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 40
    8: 0x0000000000400546    pushq  $0x2
    9: 0x000000000040054b    jmp    0x400510
  a.out`(none)
    10: 0x0000000000400510    pushq  0x200af2(%rip)            ; _GLOBAL_OFFSET_TABLE_ + 8
    11: 0x0000000000400516    jmpq   *0x200af4(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 16
    ...missing instructions
    12: (error) no memory mapped at this address: 0x00007ffff7df1950
  a.out`main + 20 at main.cpp:10
    16: 0x0000000000400674    movl   %eax, -0xc(%rbp)
  a.out`main + 23 at main.cpp:12
    20: 0x0000000000400677    movl   -0xc(%rbp), %eax
    21: 0x000000000040067a    addl   $0x1, %eax
    22: 0x000000000040067f    movl   %eax, -0xc(%rbp)
  a.out`main + 34 [inlined] inline_function() at main.cpp:4
    26: 0x0000000000400682    movl   $0x0, -0x4(%rbp)
  a.out`main + 41 [inlined] inline_function() + 7 at main.cpp:5
    27: 0x0000000000400689    movl   -0x4(%rbp), %eax
    28: 0x000000000040068c    addl   $0x1, %eax
    29: 0x0000000000400691    movl   %eax, -0x4(%rbp)
  a.out`main + 52 [inlined] inline_function() + 18 at main.cpp:6
    30: 0x0000000000400694    movl   -0x4(%rbp), %eax
  a.out`main + 55 at main.cpp:14
    31: 0x0000000000400697    movl   -0xc(%rbp), %ecx
    32: 0x000000000040069a    addl   %eax, %ecx
    33: 0x000000000040069c    movl   %ecx, -0xc(%rbp)
  a.out`main + 63 at main.cpp:16
    37: 0x000000000040069f    callq  0x400540                  ; symbol stub for: foo()
  a.out`symbol stub for: foo()
    38: 0x0000000000400540    jmpq   *0x200ae2(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 40
  libfoo.so`foo() at foo.cpp:3
    39: 0x00007ffff7bd96e0    pushq  %rbp
    40: 0x00007ffff7bd96e1    movq   %rsp, %rbp
  libfoo.so`foo() + 4 at foo.cpp:4
    41: 0x00007ffff7bd96e4    subq   $0x10, %rsp
    42: 0x00007ffff7bd96e8    callq  0x7ffff7bd95d0            ; symbol stub for: bar()
  libfoo.so`symbol stub for: bar()
    43: 0x00007ffff7bd95d0    jmpq   *0x200a4a(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 32
  libbar.so`bar() at bar.cpp:1
    44: 0x00007ffff79d7690    pushq  %rbp
    45: 0x00007ffff79d7691    movq   %rsp, %rbp
  libbar.so`bar() + 4 at bar.cpp:2
    46: 0x00007ffff79d7694    movl   $0x1, -0x4(%rbp)
  libbar.so`bar() + 11 at bar.cpp:3
    47: 0x00007ffff79d769b    movl   -0x4(%rbp), %eax
    48: 0x00007ffff79d769e    addl   $0x1, %eax
    49: 0x00007ffff79d76a3    movl   %eax, -0x4(%rbp)
  libbar.so`bar() + 22 at bar.cpp:4
    50: 0x00007ffff79d76a6    movl   -0x4(%rbp), %eax
    51: 0x00007ffff79d76a9    popq   %rbp
    52: 0x00007ffff79d76aa    retq""",
                """libfoo.so`foo() + 13 at foo.cpp:4
    53: 0x00007ffff7bd96ed    movl   %eax, -0x4(%rbp)
  libfoo.so`foo() + 16 at foo.cpp:5
    54: 0x00007ffff7bd96f0    movl   -0x4(%rbp), %eax
    55: 0x00007ffff7bd96f3    addl   $0x1, %eax
    56: 0x00007ffff7bd96f8    movl   %eax, -0x4(%rbp)
  libfoo.so`foo() + 27 at foo.cpp:6
    57: 0x00007ffff7bd96fb    movl   -0x4(%rbp), %eax
    58: 0x00007ffff7bd96fe    addq   $0x10, %rsp
    59: 0x00007ffff7bd9702    popq   %rbp
    60: 0x00007ffff7bd9703    retq""",
                """a.out`main + 68 at main.cpp:16
    61: 0x00000000004006a4    movl   -0xc(%rbp), %ecx
    62: 0x00000000004006a7    addl   %eax, %ecx
    63: 0x00000000004006a9    movl   %ecx, -0xc(%rbp)
    no more data""",
            ],
        )

        self.expect(
            "thread trace dump instructions --count 50",
            substrs=[
                """thread #1: tid = 815455
  a.out`main + 73 at main.cpp:16
    63: 0x00000000004006a9    movl   %ecx, -0xc(%rbp)
    62: 0x00000000004006a7    addl   %eax, %ecx
    61: 0x00000000004006a4    movl   -0xc(%rbp), %ecx
  libfoo.so`foo() + 35 at foo.cpp:6
    60: 0x00007ffff7bd9703    retq""",
                """59: 0x00007ffff7bd9702    popq   %rbp
    58: 0x00007ffff7bd96fe    addq   $0x10, %rsp
    57: 0x00007ffff7bd96fb    movl   -0x4(%rbp), %eax
  libfoo.so`foo() + 24 at foo.cpp:5
    56: 0x00007ffff7bd96f8    movl   %eax, -0x4(%rbp)
    55: 0x00007ffff7bd96f3    addl   $0x1, %eax
    54: 0x00007ffff7bd96f0    movl   -0x4(%rbp), %eax
  libfoo.so`foo() + 13 at foo.cpp:4
    53: 0x00007ffff7bd96ed    movl   %eax, -0x4(%rbp)
  libbar.so`bar() + 26 at bar.cpp:4
    52: 0x00007ffff79d76aa    retq""",
                """51: 0x00007ffff79d76a9    popq   %rbp
    50: 0x00007ffff79d76a6    movl   -0x4(%rbp), %eax
  libbar.so`bar() + 19 at bar.cpp:3
    49: 0x00007ffff79d76a3    movl   %eax, -0x4(%rbp)
    48: 0x00007ffff79d769e    addl   $0x1, %eax
    47: 0x00007ffff79d769b    movl   -0x4(%rbp), %eax
  libbar.so`bar() + 4 at bar.cpp:2
    46: 0x00007ffff79d7694    movl   $0x1, -0x4(%rbp)
  libbar.so`bar() + 1 at bar.cpp:1
    45: 0x00007ffff79d7691    movq   %rsp, %rbp
    44: 0x00007ffff79d7690    pushq  %rbp
  libfoo.so`symbol stub for: bar()
    43: 0x00007ffff7bd95d0    jmpq   *0x200a4a(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 32
  libfoo.so`foo() + 8 at foo.cpp:4
    42: 0x00007ffff7bd96e8    callq  0x7ffff7bd95d0            ; symbol stub for: bar()
    41: 0x00007ffff7bd96e4    subq   $0x10, %rsp
  libfoo.so`foo() + 1 at foo.cpp:3
    40: 0x00007ffff7bd96e1    movq   %rsp, %rbp
    39: 0x00007ffff7bd96e0    pushq  %rbp
  a.out`symbol stub for: foo()
    38: 0x0000000000400540    jmpq   *0x200ae2(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 40
  a.out`main + 63 at main.cpp:16
    37: 0x000000000040069f    callq  0x400540                  ; symbol stub for: foo()
  a.out`main + 60 at main.cpp:14
    33: 0x000000000040069c    movl   %ecx, -0xc(%rbp)
    32: 0x000000000040069a    addl   %eax, %ecx
    31: 0x0000000000400697    movl   -0xc(%rbp), %ecx
  a.out`main + 52 [inlined] inline_function() + 18 at main.cpp:6
    30: 0x0000000000400694    movl   -0x4(%rbp), %eax
  a.out`main + 49 [inlined] inline_function() + 15 at main.cpp:5
    29: 0x0000000000400691    movl   %eax, -0x4(%rbp)
    28: 0x000000000040068c    addl   $0x1, %eax
    27: 0x0000000000400689    movl   -0x4(%rbp), %eax
  a.out`main + 34 [inlined] inline_function() at main.cpp:4
    26: 0x0000000000400682    movl   $0x0, -0x4(%rbp)
  a.out`main + 31 at main.cpp:12
    22: 0x000000000040067f    movl   %eax, -0xc(%rbp)
    21: 0x000000000040067a    addl   $0x1, %eax
    20: 0x0000000000400677    movl   -0xc(%rbp), %eax
  a.out`main + 20 at main.cpp:10
    16: 0x0000000000400674    movl   %eax, -0xc(%rbp)
    ...missing instructions
    12: (error) no memory mapped at this address: 0x00007ffff7df1950
  a.out`(none)
    11: 0x0000000000400516    jmpq   *0x200af4(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 16
    10: 0x0000000000400510    pushq  0x200af2(%rip)            ; _GLOBAL_OFFSET_TABLE_ + 8
  a.out`symbol stub for: foo() + 11
    9: 0x000000000040054b    jmp    0x400510
    8: 0x0000000000400546    pushq  $0x2
    7: 0x0000000000400540    jmpq   *0x200ae2(%rip)           ; _GLOBAL_OFFSET_TABLE_ + 40
  a.out`main + 15 at main.cpp:10
    3: 0x000000000040066f    callq  0x400540                  ; symbol stub for: foo()
    no more data""",
            ],
        )

        self.expect(
            "thread trace dump instructions --skip 100 --forwards",
            inHistory=True,
            substrs=[
                """thread #1: tid = 815455
    no more data"""
            ],
        )

        self.expect(
            "",
            substrs=[
                """thread #1: tid = 815455
    no more data"""
            ],
        )

        self.expect(
            "thread trace dump instructions --raw --all --forwards",
            substrs=[
                """thread #1: tid = 815455
    3: 0x000000000040066f
    7: 0x0000000000400540""",
                """11: 0x0000000000400516
    ...missing instructions
    12: (error) no memory mapped at this address: 0x00007ffff7df1950
    16: 0x0000000000400674""",
                """61: 0x00000000004006a4
    62: 0x00000000004006a7
    63: 0x00000000004006a9
    no more data""",
            ],
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.dialects import arith, builtin, func, linalg, tensor
from mlir.dialects.linalg.opdsl.lang import *
from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


# CHECK-LABEL: TEST: testFill
@run
def testFill():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):
            # CHECK-LABEL: func @fill_tensor
            #  CHECK-SAME:   %[[OUT:[0-9a-z]+]]: tensor<12x?xf32>
            #  CHECK-NEXT: %[[CST:.*]] = arith.constant 0.0{{.*}} : f32
            #  CHECK-NEXT: %[[RES:.*]] = linalg.fill ins(%[[CST]] : f32) outs(%[[OUT]] : tensor<12x?xf32>) -> tensor<12x?xf32>
            #  CHECK-NEXT: return %[[RES]] : tensor<12x?xf32>
            @func.FuncOp.from_py_func(
                RankedTensorType.get((12, ShapedType.get_dynamic_size()), f32)
            )
            def fill_tensor(out):
                zero = arith.ConstantOp(
                    value=FloatAttr.get(f32, 0.0), result=f32
                ).result
                return linalg.fill(zero, outs=[out])

            # CHECK-LABEL: func @fill_buffer
            #  CHECK-SAME:   %[[OUT:[0-9a-z]+]]: memref<12x?xf32>
            #  CHECK-NEXT: %[[CST:.*]] = arith.constant 0.0{{.*}} : f32
            #  CHECK-NEXT: linalg.fill ins(%[[CST]] : f32) outs(%[[OUT]] : memref<12x?xf32>)
            #  CHECK-NEXT: return
            @func.FuncOp.from_py_func(
                MemRefType.get((12, ShapedType.get_dynamic_size()), f32)
            )
            def fill_buffer(out):
                zero = arith.ConstantOp(
                    value=FloatAttr.get(f32, 0.0), result=f32
                ).result
                linalg.fill(zero, outs=[out])

    print(module)


# CHECK-LABEL: TEST: testNamedStructuredOpCustomForm
@run
def testNamedStructuredOpCustomForm():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                RankedTensorType.get((4, 8), f32), RankedTensorType.get((4, 8), f32)
            )
            def named_form(lhs, rhs):
                init_result = tensor.EmptyOp([4, 8], f32)
                # Check for the named form with custom format
                #      CHECK: linalg.elemwise_unary
                # CHECK-SAME:    cast = #linalg.type_fn<cast_signed>
                # CHECK-SAME:    fun = #linalg.unary_fn<exp>
                # CHECK-SAME:    ins(%{{.*}} : tensor<4x8xf32>) outs(%{{.*}} : tensor<4x8xf32>)
                unary_result = linalg.elemwise_unary(lhs, outs=[init_result.result])
                #      CHECK: linalg.elemwise_binary
                # CHECK-SAME:    cast = #linalg.type_fn<cast_unsigned>
                # CHECK-SAME:    fun = #linalg.binary_fn<mul>
                # CHECK-SAME:    ins(%{{.*}}, %{{.*}} : tensor<4x8xf32>, tensor<4x8xf32>) outs(%{{.*}} : tensor<4x8xf32>)
                #      CHECK: return
                binary_result = linalg.elemwise_binary(
                    lhs,
                    rhs,
                    outs=[init_result.result],
                    fun=BinaryFn.mul,
                    cast=TypeFn.cast_unsigned,
                )
                return unary_result, binary_result

    print(module)


# CHECK-LABEL: TEST: testNamedStructuredOpGenericForm
@run
def testNamedStructuredOpGenericForm():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                RankedTensorType.get((4, 16), f32), RankedTensorType.get((16, 8), f32)
            )
            def named_form(lhs, rhs):
                init_result = tensor.EmptyOp([4, 8], f32)
                #      CHECK: "linalg.matmul"(%{{.*}})
                # CHECK-SAME:    cast = #linalg.type_fn<cast_signed>
                # CHECK-SAME:    operandSegmentSizes = array<i32: 2, 1>
                # CHECK-NEXT:  ^bb0(%{{.*}}: f32, %{{.*}}: f32, %{{.*}}: f32):
                # CHECK-NEXT:    arith.mulf{{.*}} (f32, f32) -> f32
                # CHECK-NEXT:    arith.addf{{.*}} (f32, f32) -> f32
                # CHECK-NEXT:    linalg.yield{{.*}} (f32) -> ()
                # CHECK-NEXT: (tensor<4x16xf32>, tensor<16x8xf32>, tensor<4x8xf32>) -> tensor<4x8xf32>
                return linalg.matmul(lhs, rhs, outs=[init_result.result])

    module.operation.print(print_generic_op_form=True)


# CHECK-LABEL: TEST: testNamedStructuredAsGenericOp
@run
def testNamedStructuredAsGenericOp():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                RankedTensorType.get((4, 16), f32), RankedTensorType.get((16, 8), f32)
            )
            def generic_form(lhs, rhs):
                init_result = tensor.EmptyOp([4, 8], f32)
                # CHECK: linalg.generic
                return linalg.matmul(
                    lhs, rhs, outs=[init_result.result], emit_generic=True
                )

    print(module)


# CHECK-LABEL: TEST: testOpResultFromOtherOp
@run
def testOpResultFromOtherOp():
    with Context(), Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                RankedTensorType.get((4, 16), f32), RankedTensorType.get((16, 8), f32)
            )
            def pass_an_op_directly(arg0, arg1):
                one = arith.ConstantOp(F32Type.get(), 1.0)
                # CHECK: %[[LHS:.*]] = linalg.fill
                lhs = linalg.fill(one, outs=[arg0])
                # CHECK: %[[RHS:.*]] = linalg.fill
                rhs = linalg.fill(one, outs=[arg1])
                # CHECK: %[[INIT:.*]] = tensor.empty
                init = tensor.EmptyOp([4, 8], f32)
                # CHECK: linalg.matmul
                # CHECK: ins(%[[LHS]], %[[RHS]]
                # CHECK: outs(%[[INIT]]
                return linalg.matmul(lhs, rhs, outs=init)

    print(module)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftExpressionErrorReportingy(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    def test_missing_var(self):
        """Test error reporting in expressions reports
        only diagnostics in user code"""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        options = lldb.SBExpressionOptions()
        value = self.frame().EvaluateExpression(
            "ceciNestPasUnVar", options)
        def check(value):
            lines = str(value.GetError()).split('\n')
            self.assertTrue(lines[0].startswith('error:'))
            self.assertIn('ceciNestPasUnVar', lines[0])
            for line in lines[1:]:
                self.assertFalse(line.startswith('error:'))
                self.assertFalse(line.startswith('warning:'))

        check(value)
        process.Continue()
        value = self.frame().EvaluateExpression(
            "ceciNestPasUnVar", options)
        check(value)

    @swiftTest
    def test_missing_type(self):
        """Test error reporting in expressions reports
        only diagnostics in user code"""
        self.build(dictionary={'HIDE_SWIFTMODULE': 'YES'})
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        options = lldb.SBExpressionOptions()
        value = self.frame().EvaluateExpression("strct", options)
        def check(value):
            lines = str(value.GetError()).split('\n')
            self.assertTrue(lines[0].startswith('error:'))
            self.assertIn('Missing type', lines[0])
            self.assertIn('strct', lines[0])
            for line in lines[1:]:
                self.assertFalse(line.startswith('error:'))
                self.assertFalse(line.startswith('warning:'))

        check(value)

        self.expect('dwim-print -O -- strct', error=True,
                    substrs=['Missing type'])
        
        process.Continue()
        self.expect('dwim-print -O -- number', error=True,
                    substrs=['self', 'not', 'found'])



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestWriteMemory(GDBRemoteTestBase):
    def test(self):
        class MyResponder(MockGDBServerResponder):
            def setBreakpoint(self, packet):
                return "OK"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTargetWithFileAndTargetTriple("", "x86_64-pc-linux")
        process = self.connect(target)

        bp = target.BreakpointCreateByAddress(0x1000)
        self.assertTrue(bp.IsValid())
        self.assertEqual(bp.GetNumLocations(), 1)
        bp.SetEnabled(True)
        self.assertTrue(bp.IsEnabled())

        err = lldb.SBError()
        data = str("\x01\x02\x03\x04")
        result = process.WriteMemory(0x1000, data, err)
        self.assertEqual(result, 4)



import os
import itertools
import platform
import subprocess
import sys

import lit.util
from lit.llvm import llvm_config
from lit.llvm.subst import FindTool
from lit.llvm.subst import ToolSubst


def _get_lldb_init_path(config):
    return os.path.join(config.test_exec_root, "lit-lldb-init-quiet")


def _disallow(config, execName):
    warning = """
    echo '*** Do not use \'{0}\' in tests; use \'%''{0}\'. ***' &&
    exit 1 && echo
  """
    config.substitutions.append((" {0} ".format(execName), warning.format(execName)))


def use_lldb_substitutions(config):
    # Set up substitutions for primary tools.  These tools must come from config.lldb_tools_dir
    # which is basically the build output directory.  We do not want to find these in path or
    # anywhere else, since they are specifically the programs which are actually being tested.

    dsname = "debugserver" if platform.system() in ["Darwin"] else "lldb-server"
    dsargs = [] if platform.system() in ["Darwin"] else ["gdbserver"]

    build_script = os.path.dirname(__file__)
    build_script = os.path.join(build_script, "build.py")
    build_script_args = [
        build_script,
        "--compiler=any",  # Default to best compiler
        "--arch=" + str(config.lldb_bitness),
    ]
    if config.lldb_lit_tools_dir:
        build_script_args.append("--tools-dir={0}".format(config.lldb_lit_tools_dir))
    if config.lldb_tools_dir:
        build_script_args.append("--tools-dir={0}".format(config.lldb_tools_dir))
    if config.llvm_libs_dir:
        build_script_args.append("--libs-dir={0}".format(config.llvm_libs_dir))
    if config.objc_gnustep_dir:
        build_script_args.append(
            '--objc-gnustep-dir="{0}"'.format(config.objc_gnustep_dir)
        )
    if config.cmake_sysroot:
        build_script_args.append("--sysroot={0}".format(config.cmake_sysroot))

    lldb_init = _get_lldb_init_path(config)

    primary_tools = [
        ToolSubst(
            "%lldb",
            command=FindTool("lldb"),
            extra_args=["--no-lldbinit", "-S", lldb_init],
            unresolved="fatal",
        ),
        ToolSubst(
            "%lldb-init",
            command=FindTool("lldb"),
            extra_args=["-S", lldb_init],
            unresolved="fatal",
        ),
        ToolSubst(
            "%lldb-noinit",
            command=FindTool("lldb"),
            extra_args=["--no-lldbinit"],
            unresolved="fatal",
        ),
        ToolSubst(
            "%lldb-server",
            command=FindTool("lldb-server"),
            extra_args=[],
            unresolved="ignore",
        ),
        ToolSubst(
            "%debugserver",
            command=FindTool(dsname),
            extra_args=dsargs,
            unresolved="ignore",
        ),
        ToolSubst(
            "%platformserver",
            command=FindTool("lldb-server"),
            extra_args=["platform"],
            unresolved="ignore",
        ),
        "lldb-test",
        "lldb-dap",
        ToolSubst(
            "%build", command="'" + sys.executable + "'", extra_args=build_script_args
        ),
    ]

    _disallow(config, "lldb")
    _disallow(config, "lldb-server")
    _disallow(config, "debugserver")
    _disallow(config, "platformserver")

    llvm_config.add_tool_substitutions(primary_tools, [config.lldb_tools_dir])


def _use_msvc_substitutions(config):
    # If running from a Visual Studio Command prompt (e.g. vcvars), this will
    # detect the include and lib paths, and find cl.exe and link.exe and create
    # substitutions for each of them that explicitly specify /I and /L paths
    cl = lit.util.which("cl")

    if not cl:
        return

    # Don't use lit.util.which() for link.exe: In `git bash`, it will pick
    # up /usr/bin/link (another name for ln).
    link = os.path.join(os.path.dirname(cl), "link.exe")

    cl = '"' + cl + '"'
    link = '"' + link + '"'
    includes = os.getenv("INCLUDE", "").split(";")
    libs = os.getenv("LIB", "").split(";")

    config.available_features.add("msvc")
    compiler_flags = ['"/I{}"'.format(x) for x in includes if os.path.exists(x)]
    linker_flags = ['"/LIBPATH:{}"'.format(x) for x in libs if os.path.exists(x)]

    tools = [
        ToolSubst("%msvc_cl", command=cl, extra_args=compiler_flags),
        ToolSubst("%msvc_link", command=link, extra_args=linker_flags),
    ]
    llvm_config.add_tool_substitutions(tools)
    return


def use_support_substitutions(config):
    # Set up substitutions for support tools.  These tools can be overridden at the CMake
    # level (by specifying -DLLDB_LIT_TOOLS_DIR), installed, or as a last resort, we can use
    # the just-built version.
    host_flags = ["--target=" + config.host_triple]
    if platform.system() in ["Darwin"]:
        try:
            out = subprocess.check_output(["xcrun", "--show-sdk-path"]).strip()
            res = 0
        except OSError:
            res = -1
        if res == 0 and out:
            sdk_path = lit.util.to_string(out)
            llvm_config.lit_config.note("using SDKROOT: %r" % sdk_path)
            host_flags += ["-isysroot", sdk_path]
    elif sys.platform != "win32":
        host_flags += ["-pthread"]

    config.target_shared_library_suffix = (
        ".dylib" if platform.system() in ["Darwin"] else ".so"
    )
    config.substitutions.append(
        ("%target-shared-library-suffix", config.target_shared_library_suffix)
    )

    # Swift support
    swift_args = [
        "-module-cache-path",
        os.path.join(
            os.path.dirname(config.lldb_libs_dir),
            "lldb-test-build.noindex",
            "module-cache-clang",
        ),
    ]
    swift_driver_args = []
    if platform.system() in ["Darwin"]:
        swift_args += ["-sdk", sdk_path]
    tools = [
        ToolSubst(
            "%target-swiftc",
            command=config.swiftc,
            extra_args=swift_args + swift_driver_args,
        ),
        ToolSubst(
            "%target-swift-frontend",
            command=config.swiftc[:-1],
            extra_args=(["-frontend"] + swift_args),
        ),
    ]
    llvm_config.add_tool_substitutions(tools)

    swift_bin_dir = os.path.dirname(config.swiftc)
    swift_Benchmark_Onone = os.path.join(
        swift_bin_dir, "Benchmark_Onone-{0}".format(config.target_triple)
    )
    if os.path.exists(swift_Benchmark_Onone):
        config.substitutions.append(("%swift_Benchmark_Onone", swift_Benchmark_Onone))
        config.available_features.add("swift_Benchmark_Onone")

    if sys.platform.startswith("netbsd"):
        # needed e.g. to use freshly built libc++
        host_flags += [
            "-L" + config.llvm_libs_dir,
            "-Wl,-rpath," + config.llvm_libs_dir,
        ]

    # The clang module cache is used for building inferiors.
    host_flags += ["-fmodules-cache-path={}".format(config.clang_module_cache)]

    if config.cmake_sysroot:
        host_flags += ["--sysroot={}".format(config.cmake_sysroot)]

    host_flags = " ".join(host_flags)
    config.substitutions.append(("%clang_host", "%clang " + host_flags))
    config.substitutions.append(("%clangxx_host", "%clangxx " + host_flags))
    config.substitutions.append(
        ("%clang_cl_host", "%clang_cl --target=" + config.host_triple)
    )

    additional_tool_dirs = []
    if config.lldb_lit_tools_dir:
        additional_tool_dirs.append(config.lldb_lit_tools_dir)

    llvm_config.use_clang(
        additional_flags=["--target=specify-a-target-or-use-a-_host-substitution"],
        additional_tool_dirs=additional_tool_dirs,
        required=True,
        use_installed=True,
    )

    if sys.platform == "win32":
        _use_msvc_substitutions(config)

    have_lld = llvm_config.use_lld(
        additional_tool_dirs=additional_tool_dirs, required=False, use_installed=True
    )
    if have_lld:
        config.available_features.add("lld")

    support_tools = [
        "yaml2obj",
        "obj2yaml",
        "llvm-dwp",
        "llvm-pdbutil",
        "llvm-mc",
        "llvm-readobj",
        "llvm-objdump",
        "llvm-objcopy",
        "lli",
    ]
    additional_tool_dirs += [config.lldb_tools_dir, config.llvm_tools_dir]
    llvm_config.add_tool_substitutions(support_tools, additional_tool_dirs)

    _disallow(config, "clang")


def use_lldb_repro_substitutions(config, mode):
    lldb_init = _get_lldb_init_path(config)
    substitutions = [
        ToolSubst(
            "%lldb",
            command=FindTool("lldb-repro"),
            extra_args=[mode, "--no-lldbinit", "-S", lldb_init],
        ),
        ToolSubst(
            "%lldb-init",
            command=FindTool("lldb-repro"),
            extra_args=[mode, "-S", lldb_init],
        ),
    ]
    llvm_config.add_tool_substitutions(substitutions, [config.lldb_tools_dir])



# TestSwiftBackwardsCompatibilitySimple.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test import configuration
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import swift
import unittest2


class TestSwiftBackwardsCompatibilitySimple(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    @skipIf(compiler="swiftc", compiler_version=['<', '5.0'])
    def test_simple(self):
        if configuration.swiftCompiler:
            compiler = configuration.swiftCompiler
        else:
            compiler = swift.getSwiftCompiler()
        version = self.getCompilerVersion(compiler)
        if version < '5.0':
            self.skipTest('Swift compiler predates stable ABI')
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))
        # FIXME: Removing the next line breaks subsequent expressions
        #        when the swiftmodules can't be loaded.
        self.expect("fr v")
        self.expect("fr v number", substrs=['23'])
        self.expect("fr v array", substrs=['1', '2', '3'])
        self.expect("fr v string", substrs=['"hello"'])
        self.expect("fr v tuple", substrs=['42', '"abc"'])
        # FIXME: This doesn't work.
        #self.expect("fr v generic", substrs=['-1'])




"""
Test process attach.
"""


import os
import lldb
import shutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

exe_name = "ProcessAttach"  # Must match Makefile


class ProcessAttachTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.c.
        self.line = line_number("main.cpp", "// Waiting to be attached...")

    @skipIfiOSSimulator
    def test_attach_to_process_by_id(self):
        """Test attach by process id"""
        self.build()
        exe = self.getBuildArtifact(exe_name)

        # Spawn a new process
        popen = self.spawnSubprocess(exe)

        self.runCmd("process attach -p " + str(popen.pid))

        target = self.dbg.GetSelectedTarget()

        process = target.GetProcess()
        self.assertTrue(process, PROCESS_IS_VALID)

    @skipIfiOSSimulator
    def test_attach_to_process_by_id_autocontinue(self):
        """Test attach by process id"""
        self.build()
        exe = self.getBuildArtifact(exe_name)

        # Spawn a new process
        popen = self.spawnSubprocess(exe)

        self.runCmd("process attach -c -p " + str(popen.pid))

        target = self.dbg.GetSelectedTarget()

        process = target.GetProcess()
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertTrue(process.GetState(), lldb.eStateRunning)

    @skipIfWindows  # This is flakey on Windows AND when it fails, it hangs: llvm.org/pr48806
    def test_attach_to_process_from_different_dir_by_id(self):
        """Test attach by process id"""
        newdir = self.getBuildArtifact("newdir")
        try:
            os.mkdir(newdir)
        except OSError as e:
            if e.errno != os.errno.EEXIST:
                raise
        testdir = self.getBuildDir()
        exe = os.path.join(newdir, "proc_attach")
        self.buildProgram("main.cpp", exe)
        self.addTearDownHook(lambda: shutil.rmtree(newdir))

        # Spawn a new process
        popen = self.spawnSubprocess(exe)

        os.chdir(newdir)
        self.addTearDownHook(lambda: os.chdir(testdir))
        self.runCmd("process attach -p " + str(popen.pid))

        target = self.dbg.GetSelectedTarget()

        process = target.GetProcess()
        self.assertTrue(process, PROCESS_IS_VALID)

    def test_attach_to_process_by_name(self):
        """Test attach by process name"""
        self.build()
        exe = self.getBuildArtifact(exe_name)

        # Spawn a new process
        popen = self.spawnSubprocess(exe)

        self.runCmd("process attach -n " + exe_name)

        target = self.dbg.GetSelectedTarget()

        process = target.GetProcess()
        self.assertTrue(process, PROCESS_IS_VALID)

    @skipIfWindows  # This test is flaky on Windows
    @expectedFailureNetBSD
    def test_attach_to_process_by_id_correct_executable_offset(self):
        """
        Test that after attaching to a process the executable offset
        is determined correctly on FreeBSD.  This is a regression test
        for dyld plugin getting the correct executable path,
        and therefore being able to identify it in the module list.
        """

        self.build()
        exe = self.getBuildArtifact(exe_name)

        # In order to reproduce, we must spawn using a relative path
        popen = self.spawnSubprocess(os.path.relpath(exe))

        self.runCmd("process attach -p " + str(popen.pid))

        # Make sure we did not attach too early.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=False
        )
        self.runCmd("process continue")
        self.expect("v g_val", substrs=["12345"])

    def tearDown(self):
        # Destroy process before TestBase.tearDown()
        self.dbg.GetSelectedTarget().GetProcess().Destroy()

        # Call super's tearDown().
        TestBase.tearDown(self)



"""
Test utility functions for the frame object.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FrameUtilsTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.c", "// Find the line number here.")

    def test_frame_utils(self):
        """Test utility functions for the frame object."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByLocation("main.c", self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.LaunchProcess() failed")
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        import lldbsuite.test.lldbutil as lldbutil

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(thread)
        frame0 = thread.GetFrameAtIndex(0)
        self.assertTrue(frame0)
        frame1 = thread.GetFrameAtIndex(1)
        self.assertTrue(frame1)
        parent = lldbutil.get_parent_frame(frame0)
        self.assertTrue(parent and parent.GetFrameID() == frame1.GetFrameID())
        frame0_args = lldbutil.get_args_as_string(frame0)
        parent_args = lldbutil.get_args_as_string(parent)
        self.assertTrue(frame0_args and parent_args and "(int)val=1" in frame0_args)
        if self.TraceOn():
            lldbutil.print_stacktrace(thread)
            print("Current frame: %s" % frame0_args)
            print("Parent frame: %s" % parent_args)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestSwiftMutablePersistentVar(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    @swiftTest
    def test(self):
        """Test that persistent variables are mutable."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        self.expect("expr var $count = 30")
        self.expect("expr $count = 41")
        self.expect("expr $count += 1")
        self.expect("expr $count", substrs=["42"])



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil


class TestCase(lldbtest.TestBase):
    @swiftTest
    @skipUnlessFoundation
    def test(self):
        """Check that ClangImporter options can be overridden."""
        self.build()

        log = self.getBuildArtifact("lldb.log")
        self.runCmd(f"log enable lldb types -f '{log}'")
        self.runCmd("settings set target.swift-clang-override-options x-DDELETEME=1")

        lldbutil.run_to_name_breakpoint(self, "main", bkpt_module="a.out")
        self.expect("expression 1")

        self.filecheck(f"platform shell cat {log}", __file__)
        # CHECK: CCC_OVERRIDE_OPTIONS: x-DDELETEME=1
        # CHECK: Deleting argument -DDELETEME=1




"""
Test that a C++ class is visible in Swift.
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftForwardInteropTemplateTypes(TestBase):

    @swiftTest
    def test(self):
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        # rdar://106455215 validating the typeref type system will trigger an assert in Clang
        self.runCmd('settings set symbols.swift-validate-typesystem false') 
        self.expect('v wrapped', substrs=['Wrapper<CxxClass>', 't', 'a1', '10', 'a2', '20', 'a3', '30'])

        # FIXME: rdar://106455215
        # self.expect('expr wrapped', substrs=['Wrapper<CxxClass>', 't', 'a1', '10', 'a2', '20', 'a3', '30'])



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.IsValid()
    obj.GetName()
    obj.GetExecutableFile()
    obj.GetProcessID()
    obj.GetUserID()
    obj.GetGroupID()
    obj.UserIDIsValid()
    obj.GroupIDIsValid()
    obj.GetEffectiveUserID()
    obj.GetEffectiveGroupID()
    obj.EffectiveUserIDIsValid()
    obj.EffectiveGroupIDIsValid()
    obj.GetParentProcessID()



"""
Test that if we hit a breakpoint on a lambda capture
on two threads at the same time we stop only for
the correct one.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestBreakOnLambdaCapture(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_break_on_lambda_capture(self):
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")

        (target, process, main_thread, _) = lldbutil.run_to_source_breakpoint(
            self, "First break", self.main_source_file
        )

        # FIXME: This is working around a separate bug. If you hit a breakpoint and
        # run an expression and it is the first expression you've ever run, on
        # Darwin that will involve running the ObjC runtime parsing code, and we'll
        # be in the middle of that when we do PerformAction on the other thread,
        # which will cause the condition expression to fail.  Calling another
        # expression first works around this.
        val_obj = main_thread.frame[0].EvaluateExpression("true")
        self.assertSuccess(val_obj.GetError(), "Ran our expression successfully")
        self.assertEqual(val_obj.value, "true", "Value was true.")

        bkpt = target.BreakpointCreateBySourceRegex(
            "Break here in the helper", self.main_source_file
        )

        bkpt.SetCondition("enable && usec == 1")
        process.Continue()

        # This is hard to test definitively, becuase it requires hitting
        # a breakpoint on multiple threads at the same time.  On Darwin, this
        # will happen pretty much ever time we continue.  What we are really
        # asserting is that we only ever stop on one thread, so we approximate that
        # by continuing 20 times and assert we only ever hit the first thread.  Either
        # this is a platform that only reports one hit at a time, in which case all
        # this code is unused, or we actually didn't hit the other thread.

        for idx in range(0, 20):
            process.Continue()
            for thread in process.threads:
                if thread.id == main_thread.id:
                    self.assertStopReason(
                        thread.stop_reason, lldb.eStopReasonBreakpoint
                    )
                else:
                    self.assertStopReason(thread.stop_reason, lldb.eStopReasonNone)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @swiftTest
    @skipUnlessDarwin
    def test_objc_self(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "check self", lldb.SBFileSpec("main.swift"))
        self.expect("frame variable _prop", startstr="(Int) _prop = 30")

    @swiftTest
    @skipUnlessDarwin
    def test_objc_self_capture_idiom(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "check idiomatic self", lldb.SBFileSpec("main.swift"))
        self.expect("frame variable self", startstr="(a.Classic) self = 0x")
        self.expect("frame variable _prop", startstr="(Int) _prop = 30")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftWerror(TestBase):

    NO_DEBUG_INFO_TESTCASE = True

    # Don't run ClangImporter tests if Clangimporter is disabled.
    @expectedFailureAll(setting=('plugin.typesystem.clang.experimental-redecl-completion', 'true'))
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(setting=('symbols.swift-precise-compiler-invocation', 'true'))
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        """This tests that -Werror is removed from ClangImporter options by
           introducing two conflicting macro definitions in different dylibs.
        """
        self.build()
        target,  _, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec('dylib.swift'),
            extra_images=['Dylib'])

        # Turn on logging.
        log = self.getBuildArtifact("types.log")
        self.expect('log enable lldb types -f "%s"' % log)

        self.expect("expression foo", DATA_TYPES_DISPLAYED_CORRECTLY, substrs=["42"])
        self.filecheck('platform shell cat "%s"' % log, __file__)
#       CHECK-NOT: SwiftASTContextForExpressions{{.*}}-Werror
#       CHECK:     SwiftASTContextForExpressions{{.*}}-DCONFLICT
#       CHECK-NOT: SwiftASTContextForExpressions{{.*}}-Werror
#       CHECK:     SwiftASTContextForExpressions{{.*}}-DCONFLICT
#       CHECK-NOT: SwiftASTContextForExpressions{{.*}}-DCONFLICT
#       CHECK-NOT: SwiftASTContextForExpressions{{.*}}-Werror



"""Test lldb's stepping speed."""

import sys
import lldb
from lldbsuite.test import configuration
from lldbsuite.test import lldbtest_config
from lldbsuite.test.lldbbench import *
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SteppingSpeedBench(BenchBase):
    def setUp(self):
        BenchBase.setUp(self)
        self.exe = lldbtest_config.lldbExec
        self.break_spec = "-n main"
        self.count = 50

        self.trace("self.exe=%s" % self.exe)
        self.trace("self.break_spec=%s" % self.break_spec)

    @benchmarks_test
    @no_debug_info_test
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr22274: need a pexpect replacement for windows",
    )
    def test_run_lldb_steppings(self):
        """Test lldb steppings on a large executable."""
        print()
        self.run_lldb_steppings(self.exe, self.break_spec, self.count)
        print("lldb stepping benchmark:", self.stopwatch)

    def run_lldb_steppings(self, exe, break_spec, count):
        import pexpect

        # Set self.child_prompt, which is "(lldb) ".
        self.child_prompt = "(lldb) "
        prompt = self.child_prompt

        # So that the child gets torn down after the test.
        self.child = pexpect.spawn(
            "%s %s %s" % (lldbtest_config.lldbExec, self.lldbOption, exe)
        )
        child = self.child

        # Turn on logging for what the child sends back.
        if self.TraceOn():
            child.logfile_read = sys.stdout

        child.expect_exact(prompt)
        child.sendline("breakpoint set %s" % break_spec)
        child.expect_exact(prompt)
        child.sendline("run")
        child.expect_exact(prompt)

        # Reset the stopwatch now.
        self.stopwatch.reset()
        for i in range(count):
            with self.stopwatch:
                # Disassemble the function.
                child.sendline("next")  # Aka 'thread step-over'.
                child.expect_exact(prompt)

        child.sendline("quit")
        try:
            self.child.expect(pexpect.EOF)
        except:
            pass

        self.child = None



"""
Test process attach when executable was deleted.
"""


import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDeletedExecutable(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfWindows  # cannot delete a running executable
    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Use a file as a synchronization point between test and inferior.
        pid_file_path = lldbutil.append_to_process_working_directory(
            self, "token_pid_%d" % (int(os.getpid()))
        )
        self.addTearDownHook(
            lambda: self.run_platform_command("rm %s" % (pid_file_path))
        )

        # Spawn a new process
        popen = self.spawnSubprocess(exe, [pid_file_path])

        # Wait until process has fully started up.
        pid = lldbutil.wait_for_file_on_target(self, pid_file_path)

        # Now we can safely remove the executable and test if we can attach.
        os.remove(exe)

        self.runCmd("process attach -p " + str(popen.pid))
        self.runCmd("kill")



import string
import random

random.seed(0)

minDigits = 1
maxDigits = 4

def number():
  return "".join([random.choice(string.digits) for _ in range(random.randint(minDigits,maxDigits))])

minWord = 5
maxWord = 10
def word():
  return "".join([random.choice(string.ascii_letters) for _ in range(random.randint(minWord,maxWord))])

minDice = 1
maxDice = 4
def roll():
  die = []
  for _ in range(random.randint(minDice, maxDice)):
    roll = ""
    if random.randint(0,1) == 1:
      roll += number()
      
    if random.randint(0,1) == 1:
      roll += "d" + number()
    else:
      roll += "D" + number()
      
    die.append(roll)
  return "+".join(die)


line_num = 2000
things_per_line = 10

lines = []
for _ in range(line_num):
  line = []
  for _ in range(things_per_line):
    if random.randint(0,1) == 1:
      line.append(word())
    else:
      line.append(roll())
  lines.append(" ".join(line))

dice_text = "\n    ".join(lines)

dice_rolls = []
for _ in range(2000):
  dice_rolls.append(roll())

dice_rolls_formatted = "[" + ",\n    ".join(["\"" + x + "\"" for x in dice_rolls]) + "]"

res = """
extension Inputs {{
  /// Generated by Utils/benchmark-generators/generateDiceNotation.py

  static let diceRollsInText: String = \"\"\"
  {0}
  \"\"\"
  static let diceRolls: [String] = {1}
}}
"""

print(res.format(dice_text, dice_rolls_formatted))



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.IsValid()
    obj.GetName()
    obj.FindSubSection("hello_section_name")
    obj.GetNumSubSections()
    obj.GetSubSectionAtIndex(600)
    obj.GetFileAddress()
    obj.GetByteSize()
    obj.GetFileOffset()
    obj.GetFileByteSize()
    obj.GetSectionData(1000, 100)
    obj.GetSectionType()
    obj.GetDescription(lldb.SBStream())
    for subsec in obj:
        s = str(subsec)
    len(obj)



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from ._python_test_ops_gen import *
from .._mlir_libs._mlirPythonTest import TestAttr, TestType, TestTensorValue, TestIntegerRankedTensorType


def register_python_test_dialect(context, load=True):
    from .._mlir_libs import _mlirPythonTest

    _mlirPythonTest.register_python_test_dialect(context, load)



import lldb
import threading

local_data = None


class BarrierContainer(threading.local):
    def __init__(self, before_interrupt_barrier, after_interrupt_barrier, event):
        self.event = event
        self.before_interrupt_barrier = before_interrupt_barrier
        self.after_interrupt_barrier = after_interrupt_barrier


class WelcomeCommand(object):
    def __init__(self, debugger, session_dict):
        return

    def get_short_help(self):
        return "A command that waits for an interrupt before returning."

    def check_was_interrupted(self, debugger, use_interpreter):
        if use_interpreter:
            self.was_interrupted = debugger.GetCommandInterpreter().WasInterrupted()
        else:
            self.was_interrupted = debugger.InterruptRequested()
        if local_data.event:
            self.was_canceled = local_data.event.is_set()

    def __call__(self, debugger, args, exe_ctx, result):
        """Command arguments:
        {interp/debugger} - Whether to use SBCommandInterpreter::WasInterrupted
                            of SBDebugger::InterruptRequested().
        check - Don't do the rendevous, just check if an interrupt was requested.
                If check is not provided, we'll do the lock and then check.
        poll  - Should we poll once after the rendevous or spin waiting for the
                interruption to happen.

        For the interrupt cases, the command waits serially on the barriers
        passed to it in local data, giving the test runner a chance to set the
        interrupt.  Once the barriers are passed, it waits for the interrupt
        or the event.
        If it finds an interrupt, it returns "Command was interrupted". If it gets an
        event before seeing the interrupt it returns "Command was not interrupted."
        For the "poll" case, it waits on the rendevous, then checks once.
        For the "check" case, it doesn't wait, but just returns whether there was
        an interrupt in force or not."""

        if local_data == None:
            result.SetError("local data was not set.")
            result.SetStatus(lldb.eReturnStatusFailed)
            return

        use_interpreter = "interp" in args
        if not use_interpreter:
            if not "debugger" in args:
                result.SetError("Must pass either 'interp' or 'debugger'")
                result.SetStatus(lldb.eReturnStatusFailed)
                return

        self.was_interrupted = False
        self.was_canceled = False

        if "check" in args:
            self.check_was_interrupted(debugger, use_interpreter)
            if self.was_interrupted:
                result.Print("Command was interrupted")
            else:
                result.Print("Command was not interrupted")
        else:
            # Wait here to rendevous in the test before it sets the interrupt.
            local_data.before_interrupt_barrier.wait()
            # Now the test will set the interrupt, and we can continue:
            local_data.after_interrupt_barrier.wait()

            if "poll" in args:
                self.check_was_interrupted(debugger, use_interpreter)
            else:
                while not self.was_interrupted and not self.was_canceled:
                    self.check_was_interrupted(debugger, use_interpreter)

            if self.was_interrupted:
                result.Print("Command was interrupted")
            else:
                result.Print("Command was not interrupted")

            if self.was_canceled:
                result.Print("Command was canceled")
        result.SetStatus(lldb.eReturnStatusSuccessFinishResult)
        return True



"""Test that corefiles with LC_NOTE "kern ver str" and "main bin spec" load commands works."""


import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestFirmwareCorefiles(TestBase):
    @skipIf(
        debug_info=no_match(["dsym"]),
        bugnumber="This test is looking explicitly for a dSYM",
    )
    @skipIf(archs=no_match(["x86_64", "arm64", "arm64e", "aarch64"]))
    @skipIfRemote
    @skipUnlessDarwin
    def test_lc_note_version_string(self):
        self.build()
        aout_exe_basename = "a.out"
        aout_exe = self.getBuildArtifact(aout_exe_basename)
        verstr_corefile = self.getBuildArtifact("verstr.core")
        verstr_corefile_invalid_ident = self.getBuildArtifact(
            "verstr-invalid-ident.core"
        )
        verstr_corefile_addr = self.getBuildArtifact("verstr-addr.core")
        create_corefile = self.getBuildArtifact("create-empty-corefile")
        slide = 0x70000000000
        call(
            create_corefile
            + " version-string "
            + verstr_corefile
            + " "
            + aout_exe
            + " 0xffffffffffffffff 0xffffffffffffffff",
            shell=True,
        )
        call(
            create_corefile
            + " version-string "
            + verstr_corefile_invalid_ident
            + ' "" '
            + "0xffffffffffffffff 0xffffffffffffffff",
            shell=True,
        )
        call(
            create_corefile
            + " version-string "
            + verstr_corefile_addr
            + " "
            + aout_exe
            + (" 0x%x" % slide)
            + " 0xffffffffffffffff",
            shell=True,
        )

        if self.TraceOn():
            self.runCmd("log enable lldb dyld host")
            self.addTearDownHook(lambda: self.runCmd("log disable lldb dyld host"))

        # Register the a.out binary with this UUID in lldb's global module
        # cache, then throw the Target away.
        target = self.dbg.CreateTarget(aout_exe)
        self.dbg.DeleteTarget(target)

        # First, try the "kern ver str" corefile
        target = self.dbg.CreateTarget("")
        err = lldb.SBError()
        if self.TraceOn():
            self.runCmd("script print('loading corefile %s')" % verstr_corefile)
        process = target.LoadCore(verstr_corefile)
        self.assertEqual(process.IsValid(), True)
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("target mod dump sections")
        self.assertEqual(target.GetNumModules(), 1)
        fspec = target.GetModuleAtIndex(0).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), aout_exe_basename)
        self.dbg.DeleteTarget(target)

        # Second, try the "kern ver str" corefile which has an invalid ident,
        # make sure we don't crash.
        target = self.dbg.CreateTarget("")
        err = lldb.SBError()
        if self.TraceOn():
            self.runCmd(
                "script print('loading corefile %s')" % verstr_corefile_invalid_ident
            )
        process = target.LoadCore(verstr_corefile_invalid_ident)
        self.assertEqual(process.IsValid(), True)

        # Third, try the "kern ver str" corefile where it loads at an address
        target = self.dbg.CreateTarget("")
        err = lldb.SBError()
        if self.TraceOn():
            self.runCmd("script print('loading corefile %s')" % verstr_corefile_addr)
        process = target.LoadCore(verstr_corefile_addr)
        self.assertEqual(process.IsValid(), True)
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("target mod dump sections")
        self.assertEqual(target.GetNumModules(), 1)
        fspec = target.GetModuleAtIndex(0).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), aout_exe_basename)
        main_sym = target.GetModuleAtIndex(0).FindSymbol("main", lldb.eSymbolTypeAny)
        main_addr = main_sym.GetStartAddress()
        self.assertGreater(main_addr.GetLoadAddress(target), slide)
        self.assertNotEqual(main_addr.GetLoadAddress(target), lldb.LLDB_INVALID_ADDRESS)
        self.dbg.DeleteTarget(target)

    @skipIf(
        debug_info=no_match(["dsym"]),
        bugnumber="This test is looking explicitly for a dSYM",
    )
    @skipIf(archs=no_match(["x86_64", "arm64", "arm64e", "aarch64"]))
    @skipIfRemote
    @skipUnlessDarwin
    def test_lc_note_main_bin_spec(self):
        self.build()
        aout_exe_basename = "a.out"
        aout_exe = self.getBuildArtifact(aout_exe_basename)
        create_corefile = self.getBuildArtifact("create-empty-corefile")
        binspec_corefile = self.getBuildArtifact("binspec.core")
        binspec_corefile_addr = self.getBuildArtifact("binspec-addr.core")
        binspec_corefile_slideonly = self.getBuildArtifact(
            "binspec-addr-slideonly.core"
        )

        slide = 0x70000000000

        ### Create our corefile
        # 0xffffffffffffffff means load address unknown
        call(
            create_corefile
            + " main-bin-spec "
            + binspec_corefile
            + " "
            + aout_exe
            + " 0xffffffffffffffff 0xffffffffffffffff",
            shell=True,
        )
        call(
            create_corefile
            + " main-bin-spec "
            + binspec_corefile_addr
            + " "
            + aout_exe
            + (" 0x%x" % slide)
            + " 0xffffffffffffffff",
            shell=True,
        )
        call(
            create_corefile
            + " main-bin-spec "
            + binspec_corefile_slideonly
            + " "
            + aout_exe
            + " 0xffffffffffffffff"
            + (" 0x%x" % slide),
            shell=True,
        )

        if self.TraceOn():
            self.runCmd("log enable lldb dyld host")
            self.addTearDownHook(lambda: self.runCmd("log disable lldb dyld host"))

        # Register the a.out binary with this UUID in lldb's global module
        # cache, then throw the Target away.
        target = self.dbg.CreateTarget(aout_exe)
        self.dbg.DeleteTarget(target)

        # First, try the "main bin spec" corefile
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("script print('loading corefile %s')" % binspec_corefile)
        process = target.LoadCore(binspec_corefile)
        self.assertEqual(process.IsValid(), True)
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("target mod dump sections")
        self.assertEqual(target.GetNumModules(), 1)
        fspec = target.GetModuleAtIndex(0).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), aout_exe_basename)
        self.dbg.DeleteTarget(target)

        # Second, try the "main bin spec" corefile where it loads at an address
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("script print('loading corefile %s')" % binspec_corefile_addr)
        process = target.LoadCore(binspec_corefile_addr)
        self.assertEqual(process.IsValid(), True)
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("target mod dump sections")
        self.assertEqual(target.GetNumModules(), 1)
        fspec = target.GetModuleAtIndex(0).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), aout_exe_basename)
        main_sym = target.GetModuleAtIndex(0).FindSymbol("main", lldb.eSymbolTypeAny)
        main_addr = main_sym.GetStartAddress()
        self.assertGreater(main_addr.GetLoadAddress(target), slide)
        self.assertNotEqual(main_addr.GetLoadAddress(target), lldb.LLDB_INVALID_ADDRESS)
        self.dbg.DeleteTarget(target)

        # Third, try the "main bin spec" corefile where it loads at a slide
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd(
                "script print('loading corefile %s')" % binspec_corefile_slideonly
            )
        process = target.LoadCore(binspec_corefile_slideonly)
        self.assertEqual(process.IsValid(), True)
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("target mod dump sections")
        self.assertEqual(target.GetNumModules(), 1)
        fspec = target.GetModuleAtIndex(0).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), aout_exe_basename)
        main_sym = target.GetModuleAtIndex(0).FindSymbol("main", lldb.eSymbolTypeAny)
        main_addr = main_sym.GetStartAddress()
        self.assertGreater(main_addr.GetLoadAddress(target), slide)
        self.assertNotEqual(main_addr.GetLoadAddress(target), lldb.LLDB_INVALID_ADDRESS)
        self.dbg.DeleteTarget(target)

    @skipIf(
        debug_info=no_match(["dsym"]),
        bugnumber="This test is looking explicitly for a dSYM",
    )
    @skipIf(archs=no_match(["x86_64", "arm64", "arm64e", "aarch64"]))
    @skipIfRemote
    @skipUnlessDarwin
    def test_lc_note_main_bin_spec_os_plugin(self):
        self.build()
        aout_exe = self.getBuildArtifact("a.out")
        aout_exe_basename = "a.out"
        create_corefile = self.getBuildArtifact("create-empty-corefile")
        binspec_corefile_addr = self.getBuildArtifact("binspec-addr.core")

        slide = 0x70000000000

        ### Create our corefile
        # 0xffffffffffffffff means load address unknown
        call(
            create_corefile
            + " main-bin-spec "
            + binspec_corefile_addr
            + " "
            + aout_exe
            + (" 0x%x" % slide)
            + " 0xffffffffffffffff",
            shell=True,
        )

        ## We can hook in our dsym-for-uuid shell script to lldb with this env
        ## var instead of requiring a defaults write.
        dsym_for_uuid = self.getBuildArtifact("dsym-for-uuid.sh")
        os.environ["LLDB_APPLE_DSYMFORUUID_EXECUTABLE"] = dsym_for_uuid
        if self.TraceOn():
            print("Setting env var LLDB_APPLE_DSYMFORUUID_EXECUTABLE=" + dsym_for_uuid)
        self.addTearDownHook(
            lambda: os.environ.pop("LLDB_APPLE_DSYMFORUUID_EXECUTABLE", None)
        )

        self.runCmd("settings set target.load-script-from-symbol-file true")
        self.addTearDownHook(
            lambda: self.runCmd(
                "settings set target.load-script-from-symbol-file false"
            )
        )

        dsym_python_dir = os.path.join(
            "%s.dSYM" % aout_exe, "Contents", "Resources", "Python"
        )
        os.makedirs(dsym_python_dir)
        python_os_plugin_path = os.path.join(self.getSourceDir(), "operating_system.py")
        python_init = [
            "def __lldb_init_module(debugger, internal_dict):",
            "  debugger.HandleCommand('settings set target.process.python-os-plugin-path %s')"
            % python_os_plugin_path,
        ]
        with open(os.path.join(dsym_python_dir, "a_out.py"), "w") as writer:
            for l in python_init:
                writer.write(l + "\n")

        dwarfdump_uuid_regex = re.compile("UUID: ([-0-9a-fA-F]+) \(([^\(]+)\) .*")
        dwarfdump_cmd_output = subprocess.check_output(
            ('/usr/bin/dwarfdump --uuid "%s"' % aout_exe), shell=True
        ).decode("utf-8")
        aout_uuid = None
        for line in dwarfdump_cmd_output.splitlines():
            match = dwarfdump_uuid_regex.search(line)
            if match:
                aout_uuid = match.group(1)
        self.assertNotEqual(aout_uuid, None, "Could not get uuid of built a.out")

        ###  Create our dsym-for-uuid shell script which returns aout_exe
        shell_cmds = [
            "#! /bin/sh",
            "# the last argument is the uuid",
            "while [ $# -gt 1 ]",
            "do",
            "  shift",
            "done",
            "ret=0",
            'echo "<?xml version=\\"1.0\\" encoding=\\"UTF-8\\"?>"',
            'echo "<!DOCTYPE plist PUBLIC \\"-//Apple//DTD PLIST 1.0//EN\\" \\"http://www.apple.com/DTDs/PropertyList-1.0.dtd\\">"',
            'echo "<plist version=\\"1.0\\">"',
            "",
            'if [ "$1" != "%s" ]' % (aout_uuid),
            "then",
            '  echo "<key>DBGError</key><string>not found</string>"',
            '  echo "</plist>"',
            "  exit 1",
            "fi",
            "  uuid=%s" % aout_uuid,
            "  bin=%s" % aout_exe,
            "  dsym=%s.dSYM/Contents/Resources/DWARF/%s"
            % (aout_exe, os.path.basename(aout_exe)),
            'echo "<dict><key>$uuid</key><dict>"',
            "",
            'echo "<key>DBGDSYMPath</key><string>$dsym</string>"',
            'echo "<key>DBGSymbolRichExecutable</key><string>$bin</string>"',
            'echo "</dict></dict></plist>"',
            "exit $ret",
        ]

        with open(dsym_for_uuid, "w") as writer:
            for l in shell_cmds:
                writer.write(l + "\n")

        os.chmod(dsym_for_uuid, 0o755)

        ### Now run lldb on the corefile
        ### which will give us a UUID
        ### which we call dsym-for-uuid.sh with
        ### which gives us a binary and dSYM
        ### which lldb should load!

        if self.TraceOn():
            self.runCmd("log enable lldb dyld host")
            self.addTearDownHook(lambda: self.runCmd("log disable lldb dyld host"))
        # Now load the binary and confirm that we load the OS plugin.
        target = self.dbg.CreateTarget("")

        if self.TraceOn():
            self.runCmd(
                "script print('loading corefile %s with OS plugin')"
                % binspec_corefile_addr
            )

        process = target.LoadCore(binspec_corefile_addr)
        self.assertEqual(process.IsValid(), True)
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("target mod dump sections")
            self.runCmd("thread list")
        self.assertEqual(target.GetNumModules(), 1)
        fspec = target.GetModuleAtIndex(0).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), aout_exe_basename)

        # Verify our OS plug-in threads showed up
        thread = process.GetThreadByID(0x111111111)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x111111111 after we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x222222222)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x222222222 after we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x333333333)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x333333333 after we load the python OS plug-in",
        )

        self.runCmd("settings clear target.process.python-os-plugin-path")
        self.dbg.DeleteTarget(target)



"""Test that conflicting symbols in different shared libraries work correctly"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestConflictingSymbols(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        lldbutil.mkdir_p(self.getBuildArtifact("One"))
        lldbutil.mkdir_p(self.getBuildArtifact("Two"))

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24489")
    def test_conflicting_symbols(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        environment = self.registerSharedLibrariesWithTarget(target, ["One", "Two"])

        lldbutil.run_break_set_by_source_regexp(
            self, "// break here", extra_options="-f One.c", num_expected_locations=-2
        )
        lldbutil.run_break_set_by_source_regexp(
            self, "// break here", extra_options="-f Two.c", num_expected_locations=-2
        )
        lldbutil.run_break_set_by_source_regexp(
            self, "// break here", extra_options="-f main.c", num_expected_locations=1
        )

        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # This should display correctly.
        self.expect(
            "expr (unsigned long long)conflicting_symbol",
            "Symbol from One should be found",
            substrs=["11111"],
        )

        self.runCmd("continue", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        self.expect(
            "expr (unsigned long long)conflicting_symbol",
            "Symbol from Two should be found",
            substrs=["22222"],
        )

        self.runCmd("continue", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        self.expect(
            "expr (unsigned long long)conflicting_symbol",
            "An error should be printed when symbols can't be ordered",
            error=True,
            substrs=["Multiple internal symbols"],
        )

    @expectedFailureAll(bugnumber="llvm.org/pr35043")
    @skipIfWindows  # This test is "passing" on Windows, but it is a false positive.
    def test_shadowed(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        environment = self.registerSharedLibrariesWithTarget(target, ["One", "Two"])

        lldbutil.run_break_set_by_source_regexp(
            self, "// break here", extra_options="-f main.c", num_expected_locations=1
        )

        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # As we are shadowing the conflicting symbol, there should be no
        # ambiguity in this expression.
        self.expect(
            "expr int conflicting_symbol = 474747; conflicting_symbol",
            substrs=["474747"],
        )



"""
Test type lookup command.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TypeLookupTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.mm", "// break here")

    @skipUnlessDarwin
    @skipIf(archs=["i386"])
    @skipIfDarwinEmbedded  # swift crash inspecting swift stdlib with little other swift loaded <rdar://problem/55079456>
    def test_type_lookup(self):
        """Test type lookup command."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.mm", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect("type lookup NoSuchType", substrs=["@interface"], matching=False)
        self.expect("type lookup NSURL", substrs=["NSURL"])
        self.expect("type lookup NSArray", substrs=["NSArray"])
        self.expect("type lookup NSObject", substrs=["NSObject", "isa"])
        self.expect(
            "type lookup PleaseDontBeARealTypeThatExists",
            substrs=["no type was found matching 'PleaseDontBeARealTypeThatExists'"],
        )
        self.expect(
            "type lookup MyObjCClass",
            substrs=["@interface MyObjCClass", "int x", "int y"],
        )

    @expectedFailureAll(setting=('plugin.typesystem.clang.experimental-redecl-completion', 'true'))
    @skipUnlessDarwin
    @skipIf(archs=["i386"])
    @skipIfDarwinEmbedded  # swift crash inspecting swift stdlib with little other swift loaded <rdar://problem/55079456>
    def test_type_lookup_cpp_methods(self):
        """Test type lookup command."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.mm", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)
        self.expect("type lookup MyCPPClass", substrs=["setF", "float getF"])
        self.expect("type lookup MyClass", substrs=["setF", "float getF"])
